function init = sec43_oscillator_init (idx_sim, varargin)

save_dir = sec43_oscillator_savedir (idx_sim);
if ~ exist (save_dir, 'dir'),  mkdir (save_dir);  end

init_prefix = fullfile (save_dir, ...
    sprintf ('sec43_oscillator_init_%04d', idx_sim));

init_data_file = [init_prefix '.mat'];

loaded = false;
if ~ exist (init_data_file, 'file')
    diary ([init_prefix '.log']);
    run_experiment_init_ (init_data_file, idx_sim, varargin{:});
    diary ('off');
else
    loaded = true;
    fprintf ('Simulation #%d: init data loaded\n', idx_sim);
end

init = load (init_data_file);

if loaded
    rs = init.random_stream.stream;
    rs.State = init.random_stream.state_after_init;
    RandStream.setGlobalStream (rs);
end

end % function


function run_experiment_init_ ...
    (init_data_file, idx_sim, problem, data_ref)

tic_init = tic ();  start_date = datestr (now);

% Random stream
random_stream.numstreams = 2000;
random_stream.stream = RandStream.create ('threefry4x64_20', ...
    'NumStreams', random_stream.numstreams, 'StreamIndices', idx_sim);
RandStream.setGlobalStream (random_stream.stream);


%% Reference data

% Number of levels = last level
dt = data_ref.step;
nb_dt = length(dt);

% Load "exact data" (mean and variance)
meanFunc_HF = data_ref.mean_res(:, nb_dt);
varFunc_HF = data_ref.var_res(:, nb_dt);


%% Define conformity

% Compute the probability of failure assuming normal distribution
prob_func_ref = stk_distrib_normal_cdf ...
    (meanFunc_HF, problem.thres, sqrt(varFunc_HF));
prob_glob_ref = mean (prob_func_ref(:));


%% Dimensions + unit cube

dim = problem.input_dim;  % Input dimension
dimx = dim - 1;           % Dimension of x

% Define [0; 1]^3
unit_hcube = stk_hrect (dim, problem.input_names);


%% Prior of the model

len_bnd = diff(unit_hcube); % We work with data scaled in [0; 1]^d
len_bnd = len_bnd(:);

range_out = 40;             % Approximative output range

% Prior of log(sig2)
m_p_logsig2 = 2*(log(range_out) - log(100));
s_p_logsig2 = log(100);

% Prior of log(ESR)
m_p_logESR = 2*log(1);
s_p_logESR = log(100);

% Prior of -log(rho) (0 and E)
m_p_logacc = log(2) - log(len_bnd(1:dimx));
s_p_logacc = repmat(log(10), dimx, 1);

% Prior of log(nu) (0 and E)
reg = 2.5;
m_p_logreg = log(reg);
s_p_logreg = 0;

% Prior of log(L)
m_p_logdegr = log(4);
s_p_logdegr = log(3);

% Intercept
inter = 0;
s_inter = 0;

% Useless parameters (-log(rho_L) and log(nu_L))
m_p_logaccL = 0;
m_p_logregL = log(1/2);
s_p_logaccL = 0;
s_p_logregL = 0;

% Prior of log(lambda)
m_p_lognv = 2*(log(range_out) - log(100));
s_p_lognv = log(100);
c_p_lognv = 90/100;


%% Fully-Bayesian approach

% Number of parameters
nb_sim_par = 100;
% nb_sim_par_surCrit = 1: The MAP parameters

% Parameters of the Haario algorithm
time_no_adapt = 500;
burn_in       = 1000;
delay         = 100;

epsilon         = [];
type_adapt      = 'smooth';
display_waitbar = false;  % Set this to true to display Haario's waitbar

% Parameter of the Sequential Monte-Carlo sampling
nb_step = 20;


%% SUR criteria
% Number of proposal points
nc = 2500;
% Use fmincon/fminsearch
do_optim = true;

% Number of intergration points
nint = 1000;

% Pruning
frac_prun = 90/100;
ns_max    = 500;

% Load outside values (suffix `_out`) which are not sliced
bndx = problem.input_domain(:, 1:dimx);


%% Set initial desings

% Learning design: building the initial design
ni = fliplr(cumprod ([5, 2, 2, 3, 3]));
niter = 1000;

% Create a NLHS
xi = stk_sampling_nestedlhs (ni, dimx, bndx, niter, dt);
xi.colnames = problem.input_names;      % Assert column names

% Launch the simulator
zi = damped_harmonic_simulator (xi.w0, xi.zeta, xi.dt);
zi = stk_dataframe (zi, {'MaxX'});

% Rescale initial design in [0; 1]
xi_init_01 = stk_rescale (xi, problem.input_domain, unit_hcube);
% The same initial design will be used


%% Build parameters

% Covariance parameter
cov_multifid = 'stk_multifidcov_brown';
covar = stk_covmodel_multifidelity(cov_multifid, dimx, reg, inter, reg);

% Covariance prior
m_p_cov = [m_p_logsig2; m_p_logacc; m_p_logreg;
    m_p_logESR; m_p_logacc; m_p_logreg;
    m_p_logaccL; m_p_logregL; m_p_logdegr; inter];

v_p_cov = [s_p_logsig2; s_p_logacc; s_p_logreg;
    s_p_logESR; s_p_logacc; s_p_logreg;
    s_p_logaccL; s_p_logregL; s_p_logdegr; s_inter].^2;

cov_prior = stk_prior_gauss(m_p_cov, v_p_cov);
covar.prior = cov_prior;

% Log-noise variance parameter
noivar = stk_noisemodel_multilev(dt(:), NaN(nb_dt, 1), true,...
    m_p_lognv, (1 - c_p_lognv)*(s_p_lognv.^2), c_p_lognv*(s_p_lognv.^2));

% Fixed parameters
is_fixed = ([v_p_cov; diag(noivar.prior.covariance)] == 0);

% Model
model = stk_model(covar.func, dim);
model.param = covar;
model.lm = stk_lm_constant();
model.lognoisevariance = noivar;


%% Estimate parameters by maximum a posteriori

% Initialization
[cov, lnv] = stk_param_init(model, xi_init_01, zi, unit_hcube, true);

% Estimation
[cov, lnv] = stk_param_estim(model, xi_init_01, zi,...
    cov, lnv, @stk_param_mlogpost);

model.param = cov;
model.lognoisevariance = lnv;

% The fixed parameters
param_vect = stk_get_optimizable_parameters(model);

param_fixed = NaN(size(param_vect));
param_fixed(is_fixed) = param_vect(is_fixed);

% Seed of the adaptive metropolis
sim_par0 = param_vect(~is_fixed, 1);

% The distribution of parameters
distrib = @(p)(stk_haario_logpost(model, xi_init_01, zi, p, param_fixed));
isLogDistrib = true;

% Box of parameters
[lb_cov, ub_cov] = stk_param_getdefaultbounds(model.param,...
    xi_init_01, zi);
[lb_lnv, ub_lnv] = stk_param_getdefaultbounds(model.lognoisevariance,...
    xi_init_01, zi);

bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];
bnd_param = bnd_param(:, ~is_fixed);

% Initial covariance of parameters
cov_par0 = stk_haario_suggest_C0(distrib, bnd_param,...
    sim_par0, isLogDistrib);

% Haario's sampling
sim_par_init = stk_haario_adaptive_metropolis(distrib, ...
    bnd_param, nb_sim_par, sim_par0, cov_par0,...
    time_no_adapt, burn_in, delay, isLogDistrib,...
    epsilon, type_adapt, display_waitbar);

% Simulated parameters
simul_param = repmat (param_fixed', nb_sim_par, 1);
simul_param(:, ~is_fixed) = sim_par_init;

t_init = toc (tic_init);

fprintf ('Simulation #%d: init completed in %.2f hours\n', ...
    idx_sim, t_init / 3600);

random_stream.state_after_init = random_stream.stream.State;

% Info struct
info = get_config_info ();
info.start_date = start_date;  clear start_date
info.end_date = datestr (now);

% Save local workspace to file
save (init_data_file);

end % function


%#ok<*NASGU,*STRNU>
