close all;  clc;  tic_global = tic ();


%% Make it possible to specify key parameters from the outside

% Choose experiment number (see below)
if ~ exist ('ALGORITHM', 'var')
    ALGORITHM = 100;
end
fprintf ('ALGORITHM = %d\n', ALGORITHM);

% Choose number of parallel workers
% (CAUTION: Remove parfor if you do not want the PCT to start at all)
if ~ exist ('nb_workers', 'var')
    nb_workers = 10;
end
fprintf ('nb_workers = %d\n', nb_workers);

% Choose which simulations to run (simulation "id")
if ~ exist ('simulation_list', 'var')
    simulation_list = 1:(nb_workers * 2);
end

% Choose total buget as a multiple of the HF simulation time
if ~ exist ('budget_ratio', 'var')
    budget_ratio = 20;
end
fprintf ('budget_ratio = %d\n', budget_ratio);


%% Main parameters of the numerical experiment

algorithm = struct (           ...
    'name',        '',         ...
    'number',      ALGORITHM,  ...
    'type',        '',         ...
    'dt_SUR',      '',         ...
    'batch_size',  []           );

dt_SUR_list = [1/6 0.1 0.05 0.02 0.01];

switch ALGORITHM
    
    case 100  % Fully sequential MR-SUR w/ dt_pred = 0.01 (HF)
        algorithm.type = 'MRSUR';
        algorithm.dt_SUR = [];
        algorithm.dt_pred = 0.01;
        algorithm.batch_size = 1;
        
    case 130  % Batch-sequential MR-SUR w/ q = 5 and dt_pred = 0.01 (HF)
        algorithm.type = 'MRSUR';
        algorithm.dt_SUR = [];
        algorithm.dt_pred = 0.01;
        algorithm.batch_size = 5;
        
    case 124  % Fully sequential MR-SUR w/ dt_pred = 0.02 (HF)
        algorithm.type = 'MRSUR';
        algorithm.dt_SUR = [];
        algorithm.dt_pred = 0.02;
        algorithm.batch_size = 1;
        
    case 123  % Fully sequential MR-SUR w/ dt_pred = 0.05 (HF)
        algorithm.type = 'MRSUR';
        algorithm.dt_SUR = [];
        algorithm.dt_pred = 0.05;
        algorithm.batch_size = 1;
        
    case {101, 102, 103, 104, 105}  % Fully sequential fixed-level SUR
        %---------------------------- with prediction at dt = dt_HF
        algorithm.type = 'SUR';  % "fixed-level" SUR strategies
        algorithm.dt_SUR = dt_SUR_list(ALGORITHM - 100);
        algorithm.dt_pred = 0.01;
        algorithm.batch_size = 1;
        
    case {111, 112, 113, 114}       % Fully sequential fixed-level SUR
        %---------------------------- with prediction at dt = dt_SUR
        algorithm.type = 'SUR';  % "fixed-level" SUR strategies
        algorithm.dt_SUR = dt_SUR_list(ALGORITHM - 110);
        algorithm.dt_pred = algorithm.dt_SUR;
        algorithm.batch_size = 1;
        
    case {131, 132, 133, 134, 135}  % Batch-sequential fixed-level SUR
        %---------------------------- w/ q = 5 and dt_pred = dt_SUR
        algorithm.type = 'SUR';  % "fixed-level" SUR strategies
        algorithm.dt_SUR = dt_SUR_list(ALGORITHM - 130);
        algorithm.dt_pred = algorithm.dt_SUR;
        algorithm.batch_size = 5;
        
    otherwise
        error ('Incorrect ALGORITHM.');
end

% Construct algorithm name
if strcmp (algorithm.type, 'SUR')
    s1 = sprintf ('_dt=%.3f', algorithm.dt_SUR);
else
    s1 = '';
end
s2 = sprintf ('_bsize=%d', algorithm.batch_size);
algorithm.name = [algorithm.type s1 s2];


%% Problem description

problem = [];

% Load reference data
data_ref_dir = fullfile (sec43_oscillator_srcdir (), 'simulator');
data_ref = load (fullfile (data_ref_dir, '2DExample.mat'));

problem.dt = data_ref.step;
problem.dt_HF = min (problem.dt);  % High-fidelity level
problem.thres = -3;                % Safety threshold

problem.input_dim = 3;
problem.input_names = {'w0', 'zeta', 'dt'};
problem.input_domain = stk_hrect([0, 0, 0; 30, 1, 1;], problem.input_names);

problem.CPUTime = @sec43_dho_cputime;
problem.design_cost = @(x)(sum (problem.CPUTime (x.dt)));
problem.budget = problem.CPUTime (problem.dt_HF) * budget_ratio;


%% Parallel computing

if nb_workers > 1
    pool = parpool (nb_workers);
    disp_info = false;
else
    pool = [];
    disp_info = true;
end

% Number of simulations
nb_simulations = length (simulation_list);


%% Parallel loops

res = repmat (sec43_oscillator_create_res (), 1, nb_simulations);

state = struct (                    ...
    'warning',  warning ('query'),  ...
    'stk',      stk_options_get ()  );

parfor ktest = 1:nb_simulations
    
    % Restore state inside each worker
    warning (state.warning); %#ok<PFBNS>
    stk_options_set (state.stk);
    assert (isa (stk_options_get ('stk_param_estim', 'minimize_box'), 'stk_optim_fmincon'));
    
    idx_sim = simulation_list(ktest);
    
    % Results are saved inside this function:
    init = sec43_oscillator_init (idx_sim, problem, data_ref);
    
    % Results are saved inside this function too:
    res(ktest) = sec43_oscillator_seq ...
        (idx_sim, algorithm, init, problem, data_ref, disp_info);
    
end

t_global = toc (tic_global);

fprintf ('Total run time: %.2f hours\n', t_global / 3600);

% Close cluster
if ~ isempty (pool)
    delete (pool);
end
