function M = damped_harmonic_simulator(w0, zeta, dt, t_end, S, X0, V0, method)
% DAMPED_HARMONIC_SIMULATOR simulates a damped harmonic oscillator, and
% return the absolute maximum of the results.
%
% CALL : M = damped_harmonic_simulator(w0, zeta, dt)
%
%   simulates hamornic oscillators with pulsation w0 and damping ration
%   zeta, for a step time dt. Other parameters are set to there default
%   values. w0 should be in [1; 30], and must be positive. zeta should be
%   in [0, 1], and must be positive. dt should be lower than 1, and must be
%   positive.
%   All parameters must have the same size, or be a scalar (in this case,
%   the same parameters will be used in all simulations)
%
% CALL : M = damped_harmonic_simulator(w0, zeta, dt, t_end, S, X0, V0, method)
%
%   controls parameters :
%   - t_end : the end of the simulation (in s). Default : 30 s;
%   - S : the spectral intensity of the white noise. Default : 1;
%   - X0, V0 : the initial conditions. Default : 0;
%   - method : a string indicating which method to use for solving
%              equations. Default : 'exponential';
%
% See also : linear_oscillator

if nargin < 3
    error('Not enough input arguments');
end

if nargin > 8
    error('Too many input arguments');
end

%% Default arguments

% Time horizon (in second)
if nargin < 4 || isempty(t_end)
    t_end = 30;
end

% Spectral intensity of the white noise
if nargin < 5 || isempty(S)
    S = 1;
end

% Initial position
if nargin < 6 || isempty(X0)
    X0 = 0;
end

% Initial speed
if nargin < 7 || isempty(V0)
    V0 = 0;
end

% Method
if nargin < 8 || isempty(method)
    method = 'exponential';
else
    method = lower(method);
end

%% Check arguments
% Positivity
if any(w0(:) < 0) || any(zeta(:) < 0) || any(dt(:) < 0)...
        || any(t_end(:) < 0) || any(S(:) < 0)
    error('Parameters must be positive.');
end

% Size
[w0, zeta, dt, t_end, S, X0, V0] = local_stk_commonsize(w0, zeta, dt, t_end, S, X0, V0);

%% Simulations
M = NaN(size(w0));

for k = 1:numel(M)
    X = linear_oscillator(w0(k), zeta(k), dt(k), t_end(k), S(k), X0(k), V0(k), method);
    M(k) = log( max(abs(X)) );
end

end

% Replace the function stk_commonsize, to have an independent file
function varargout = local_stk_commonsize (varargin)

n = length (varargin);

if n == 0,
    
    varargout = {};
    
else
    
    if n == 1
        % varargin{1} is expected to be a cell array in this case
        C = varargin{1};
        n = length (C);
    else
        C = varargin;
    end
    
    d = cellfun (@ndims, C);
    s = ones (n, max (d));
    for i = 1:n,
        s(i, 1:d(i)) = size (C{i});
    end
    
    smax = max (s);
    
    % Take care of empty dimensions, if any
    b = (smax > 0);
    smax = smax (b);
    s = s(:, b);
    
    nrep = ones (size (smax));
    for i = 1:n,
        nrep(b) = smax ./ s(i, :);
        nrep_one = (nrep == 1);
        if ~ all ((s(i, :) == 1) | nrep_one)
            error ('Input arguments cannot be brought to a common size.');
        elseif ~ all (nrep_one)
            C{i} = repmat (C{i}, nrep);
        end
    end
    
    if nargout > 1,
        varargout = C(1:nargout);
    else
        varargout = {C};
    end
    
end

end
