% STK_SMC_EFFECT computes the effective number of weights.
%
% CALL: NEFF = stk_smc_effect(WEIGHTS)
%
%   computes the effective number NEFF of particles weighted by WEIGHTS.
%   More exactly, it computes:
%       NEFF = length(WEIGHTS)*[1 - var(WEIGHTS)/esp(WEIGHTS.^2)]
%   Remark : 1 <= NEFF <= length(WEIGHTS).
%   This function does not suppose that WEIGHTS are normalized.
%
% EXAMPLE
%   n = 1000;
%   w = rand(n, 1); wn = w/sum(w);  % Random weights
%   Neff = stk_smc_effect(w)
%   Neffn = stk_smc_effect(wn)      % Neff == Neffn
%
% See also: stk_smc_weights
function Neff = stk_smc_effect(weights)

if nargin > 1
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 1 || isempty(weights)
    Neff = 0;
else
    p = size(weights, 1);
    Neff = p*(mean(weights).^2)./mean(weights.^2);
end
end

%!shared p, n, w
%! p = randi (100, 1); n = randi(10, 1); w = rand(p, 1);
%!assert( stk_isequal_tolabs(stk_smc_effect(ones(p, n)/p), p*ones(1, n)))
%!assert( stk_isequal_tolabs(stk_smc_effect(ones(p, n)  ), p*ones(1, n)))
%!assert( stk_isequal_tolabs(stk_smc_effect([rand(1, n); zeros(p, n)]  ), ones(1, n)))
%!assert( stk_isequal_tolabs(stk_smc_effect(w), stk_smc_effect(w/sum(w))) )
