% STK_SMC_SAMPLE samples the parameters according to weights
%
% CALL: SAMP_PARAM = stk_smc_sample(SIM_PARAM, WEIGHTS)
%
%   samples the parameters SIM_PARAM according to WEIGHTS. If WEIGHTS(i) is
%   greater than k/size(SIM_PARAM, 1), the i-th parameter will be selected
%   at least k times. The remaining is selected randomly.
%
%   size(SAMP_PARAM) == size(SIM_PARAM).
%
% EXAMPLE
%   n = 10;
%   % Some weights
%   w = [0.5; 2.0; 0.5; 1.6; 0.7; 1.6; 1.2; 0.2; 1.2; 0.5;];
%   w = w/sum(w);           % Normalization
%   p = [(1:n)', -(1:n)'];	% Some 'simulated parameters'
%   q = stk_smc_sample(p, w);
%   % q(1:6, :) = p([2, 4, 6, 7, 9, 2], :)
%   % q(7:10) are randomly selected into the parameter p, with probability
%   % Parameters:   1, 2, 3, 4, 5, 6, 7, 8, 9, 10
%   % Probability: [5, 0, 5, 6, 7, 6, 2, 2, 2, 5 ]/40 (40: assert sum == 1)
%
% See also: stk_smc_effect, stk_smc_weights, stk_smc_move
function [samp_param, weights] = stk_smc_sample(sim_param, weights_param)

if nargin > 2
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 2
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Check inputs

if ~isvector(weights_param)
    stk_error('Weights must be a vector.', 'IncorrectSize');
end

weights_param = weights_param(:);   % assert column vector
nb_param = size(sim_param, 1);
% Assert size
if nb_param ~= length(weights_param)
    stk_error('Number of parameters does not equal length of weights', 'IncorrectSize');
end

% Assert are weights.
if any(weights_param(:) < 0)
    stk_error('Weights must be positive.', 'InvalidArgument');
end

if ~stk_isequal_tolabs(sum(weights_param), 1)
    warning('STK:stk_smc_sample:normalizedParam', 'Weights must be normalized.');
    weights_param = weights_param/sum(weights_param);
end

%% Find index
% List of potential values
list = (1:nb_param)';

% Index of selected parameters
ind_select = NaN(0, 1);

% If x > 0, then the parameters will be selected
x_weights = floor(weights_param*nb_param);
% Weights of the random selection
z_weights = weights_param - x_weights/nb_param;
z_weights = z_weights/sum(z_weights);
z_weights = [0; cumsum(z_weights)];

% Deterministic selection
while any(x_weights > 0)
    ind_select = cat(1, ind_select, list(x_weights > 0));
    x_weights = x_weights - 1;
end

%Random selection
rnd = rand(nb_param - length(ind_select), 1);
[Z, RND] = meshgrid(z_weights, rnd);

ind_select = [ind_select; sum( (Z - RND) < 0, 2)];

%% Return sampled parameters
samp_param = sim_param(ind_select, :);
weights = ones(nb_param, 1)/nb_param;
end

