% STK_SMC_WEIGHTS computes the weights associated to simulated parameters
%
% CALL: WEIGHTS_NEW = stk_smc_weights(SIM_PARAM, MODEL, XI, ZI, X_NEW,
%   Z_NEW, WEIGHTS_I)
%
%   updates the weights associated to the sampled parameters. SIM_PARAM
%   must be hyper-parameters of the model, sampled according to a posterior
%   distribution of hyper-parameters with observations (XI, ZI).
%   WEIGHTS_I are the current weights associated to SIM_PARAM.
%   (X_NEW, Z_NEW) are new observations.
%   The outputs WEIGHTS_NEW is normalized ( sum(WEIGHTS_NEW) == 1 ).
%
% CALL: [WEIGHTS_NEW, RELIK_NEW] = stk_smc_weights(SIM_PARAM, MODEL, XI, ZI,
%   X_NEW, Z_NEW, WEIGHTS_I, RELIK_I)
%
%   uses the RELIK_I of observations (XI, ZI) to accelerate the algorithm.
%   RELIK_I is a vector corresponding to Anti-Restricted likelihood.
%   One value RELIK_I(k) corresponds to one parameter sim_param(k, :).
%
% CALL: WEIGHTS_NEW = stk_smc_weights(SIM_PARAM, MODEL, XI, ZI, X_NEW, Z_NEW)
% CALL: [WEIGHTS_NEW, RELIK_NEW] = stk_smc_weights(SIM_PARAM, MODEL, XI, ZI,
%   X_NEW, Z_NEW, [], RELIK_I)
%
%   uses the default values for weights: 1/nb_param*ones(nb_param, 1),
%	where nb_param is the number of parameters, size(SIM_PARAM, 1).
%
% See also: stk_smc_sample, stk_smc_move
function [weights_param, relik_new] = stk_smc_weights(sim_param, model,...
    xi, zi, x_new, z_new, weights, relik_old)

if nargin > 8
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 6
    stk_error('Not enough input arguments', 'NotEnoughInputArgs');
end

%% Check inputs arguments

nb_param = size(sim_param, 1);
dim_param = size(sim_param, 2);
dim = size(xi, 2);

% Model
if model.dim ~= dim
    stk_error('The inputs and the model do not have the same dimension.',...
        'IncorrectSize');
end

% Check dimension of parameters
% Number of covariance parameters (assume there is always
% covariance parameters to optimize)
nb_param_cov = length(stk_get_optimizable_parameters(model.param));

if stk_isnoisy(model) && ~isnumeric(model.lognoisevariance)
    % Case where the log-noise variance is an object
    nb_param_lnv = length(stk_get_optimizable_parameters(model.lognoisevariance));
elseif stk_isnoisy(model) && isnumeric(model.lognoisevariance)...
        && isscalar(model.lognoisevariance)
    % Standard case of STK: homoscedastic noise described by an unique
    % parameters
    nb_param_lnv = 1;
else
    % 2 possibilities: noiseless case or heteroscedastic case
    nb_param_lnv = 0;
end
% Check dimension of parameters
if dim_param ~= (nb_param_cov + nb_param_lnv)
    stk_error(['The dimension of the simulated parameters must fit the number of',...
        ' parameters in the model.'], 'IncorrectSize');
end

% zi
if ~isequal(size(zi), [size(xi, 1), 1])
    stk_error(['zi must be a column vector with the same length as',...
        ' size(xi, 1).'], 'IncorrectSize');
end

% x_new
if size(x_new, 2) ~= dim
    stk_error('x_new and xi do not have the same dimension', 'IncorrectSize');
end

% z_new
if ~isequal(size(z_new), [size(x_new, 1), 1])
    stk_error(['z_new must be a column vector with the same length as',...
        ' size(x_new, 1).'], 'IncorrectSize');
end

% Weights
if nargin < 7 || isempty(weights)
    weights = 1/nb_param*ones(nb_param, 1);
else
    if ~isequal(size(weights), [nb_param, 1])
        stk_error(['weights must be a column vector with the same length as',...
            ' size(sim_param, 1).'], 'IncorrectSize');
    end
end

% Relik_i
if nargin < 8 || isempty(relik_old)
    is_relik_i = false;
else
    if ~isequal(size(relik_old), [nb_param, 1])
        stk_error(['relik_i must be a column vector with the same length as',...
            ' size(sim_param, 1).'], 'IncorrectSize');
    end
    is_relik_i = true;
end


%% Compute weights
xm = [xi; x_new];
zm = [zi; z_new];

relik_new = NaN(nb_param, 1);
if ~is_relik_i
    relik_old = NaN(nb_param, 1);
end

for kpar = 1:nb_param
    % 1 - change the parameters of the model
    % -> TODO: use stk_set_optimizable_parameters when available for model
    
    model.param = stk_set_optimizable_parameters(model.param,...
        sim_param(kpar, 1:nb_param_cov));
    if nb_param_lnv > 0
        model.lognoisevariance = stk_set_optimizable_parameters(model.lognoisevariance,...
            sim_param(kpar, nb_param_cov + (1:nb_param_lnv)));
    end
    
    % 2 - compute minus restricted-likelihood
    relik_new(kpar, 1) = stk_param_relik(model, xm, zm);
    
    if ~is_relik_i
        relik_old(kpar, 1) = stk_param_relik(model, xi, zi);
    end
end

% pi(theta|chi_new)/pi(theta|chi_old)
% = pi(chi_new|theta)*pi(theta)/pi(chi_old|theta)/pi(theta)
% = pi(chi_new|theta)/pi(chi_old|theta)
% = exp[ log(pi(chi_new|theta)/pi(chi_old|theta)) ]
% = exp[ log(pi(chi_new|theta)) - log(pi(chi_old|theta)) ]
% = exp[ {-log(pi(chi_old|theta))} - {-log(pi(chi_new|theta))}]

coeffs = exp(relik_old - relik_new);

% 3 - update the weights
weights_param = coeffs.*weights;
% Normalization
weights_param = weights_param/sum(weights_param);

end