% This file tests the Haario algorithm on standard 2D distribution.
% Proposed by Haario himself.
clear all; close all;
%% Check if function works

for ktest = 1:3
    % clear all;  % assert no impact between loop
    
    %% Graphical options
    % colormap
    colorm = 'parula';
    
    nx = 100;   % number of points per axis
    nf = 100;   % number of simulated points
    
    %% Define the distribution to test
    switch ktest
        case 1 % Uniform distribution
            name_distribution = 'Uniform distribution';
            % Parameter
            bnd = [
                -5, 2;
                7, 4;];
            % Distribution
            distrib = @(x)(unif_distrib(x, bnd));
            isLogDistrib = false;
            
            % Plot the real distribution
            kr = 5;	% Define the rounds of the bounds
            
        case 2 % Gaussian distribution
            name_distribution = 'Normal distribution';
            
            % Parameters of the normal distribution
            mu = [
                2;
                5;];	% mean
            sigma = [
                6, 2;
                2, 4;]; % covariance
            
            % Distribution
            distrib = @(x)(gauss_distrib(x, mu, sigma));
            isLogDistrib = false;
            
            % Define parameters bounds
            eigenvals = eig(sigma);
            major_axis = sqrt(1/eigenvals(1));
            % 5-sigma interval
            bnd = [mu' - 5*major_axis; mu' + 5*major_axis];
            
            % Plot the real distribution
            kr = 5*ceil(major_axis);	% Define the rounds of the bounds
            
        case 3 % Twisted
             name_distribution = 'Twisted Gaussian distribution';
             
            % Parameters
            mu = [
                2;
                5;];    %mean
            sigma = [
                10, 0;
                0, 1;];  %covariance
            deform = [sigma(1, 1); 0.1];
            
            % Distribution
            distrib = @(x)(twist_distrib(x, mu, sigma, deform)); % the pdf
            isLogDistrib = false;
            
            % Define parameters bounds
            eigenvals = eig(sigma);
            major_axis = sqrt(1/eigenvals(1));
            % 5-sigma interval
            bnd = [mu' - 5*major_axis; mu' + 5*major_axis];
            
            % Plot the real distribution
            kr = 5*ceil(major_axis);	% Define the rounds of the bounds
        otherwise
            stk_error('Unknown case', 'ProgrammerFailure');
    end
    
    %% Sampling
    
    % Initialize the covariance of the proposal distribution
    C0 = stk_haario_suggest_C0(distrib, bnd, [], isLogDistrib);
    
    % Sample parameters
    [xf, acc] = stk_haario_adaptive_metropolis(distrib, bnd, nf, [], C0,...
         [], [], [], isLogDistrib, [], 'hard', true);
    % Compute the auto-correlation of the chains
    autocorr = stk_haario_autocorr(xf, max(round(nf/4), 1));
    max_corr1 = max(abs(autocorr(2, :)));
    % Display the measures of quality of the sampling
    display(sprintf('%s: acceptance ratio = %.1f%%; maximal auto-correlation = %.1f%%.',...
        name_distribution, 100*acc, 100*max_corr1));
    
    %% True distribution
    % Bounds of the plot
    bnd_plot = [
        floor(bnd(1, :)/kr)*kr;
        ceil(bnd(2, :)/kr)*kr;];
    
    % x and y axis
    x = linspace(bnd_plot(1, 1), bnd_plot(2, 1), nx)';
    y = linspace(bnd_plot(1, 2), bnd_plot(2, 2), nx)';
    
    % Both together
    [Y, X] = meshgrid(y, x);
    xy = [X(:), Y(:)];
    % xy = double( stk_sampling_regulargrid(nx^2, 2, bnd_plot) );
    % Compute the distribution
    true_distrib = reshape(distrib(xy), nx, nx)';
    
    %% Figure
    
    % Plot results
    stk_figure(['Test on ', name_distribution]);
    
    % Real distribution
    stk_subplot(1, 2, 1);
    contourf(x, y, true_distrib);
    % Color
    colorbar;
    colormap(colorm);
    % Axis
    xlim(bnd_plot(:, 1));
    ylim(bnd_plot(:, 2));
    stk_xlabel('x');
    stk_ylabel('y');
    stk_title('The real distribution');
    
    % Simulations
    stk_subplot(1, 2, 2);
    hold on;
    % The simulated parameters
    plot(xf(:, 1), xf(:, 2), 'bo', 'MarkerFaceColor', 'b');
    % Contour of the distribution
    if ~isempty(strfind(name_distribution, 'Uniform'))
        % Particular plot for uniform distribution
        plot( [bnd(1, 1), bnd(2, 1), bnd(2, 1), bnd(1, 1), bnd(1, 1)],...
            [bnd(1, 2), bnd(1, 2), bnd(2, 2), bnd(2, 2), bnd(1, 2)], 'k-');
    else
        % Hold on contours
        contour(x, y, true_distrib);
    end
    % Axis
    xlim(bnd_plot(:, 1));
    xlabel('x');
    ylim(bnd_plot(:, 2));
    ylabel('y');
    stk_title('The sampled points');
end