%STK_HAARIO_LOGPOST computes the log posterior of the model.
%
% CALL: LOGPOST = stk_haario_logpost(MODEL, XI, ZI)
%
%   returns -stk_param_mlogpost(MODEL, XI, ZI), i.d., the logarithm of the
%   posterior distribution of MODEL observed at (XI, ZI).
%
% CALL: LOGPOST = stk_haario_logpost(MODEL, XI, ZI, NEW_PARAM)
%
%   changes the parameters of MODEL (covariance and noise variance, if
%   noise variance is not -Inf) before computing the log-posterior
%   distributions.
%
% CALL: LOGPOST = stk_haario_logpost(MODEL, XI, ZI, NEW_PARAM, FIXED_PARAM)
%
%   also changes the parameters, but some of them are fixed. FIXED_PARAM
%   must be a vector, whose size is the same as the number of parameters in
%   MODEL:
%       length(FIXED_PARAM) = length(stk_get_optimizable_parameters(MODEL)).
%   and whose the number of NaN is equal to the length of NEW_PARAM:
%       sum(isnan(FIXED_PARAM)) = length(NEW_PARAM).
%   The not-NaN elements of FIXED_PARAM are fixed and are not changed
%   during an Haario's sampling. The NaN element are replaced by
%   NEW_PARAM.
%
% See also: stk_param_mlogpost
function logpost = stk_haario_logpost(model, xi, zi, param_changed, param_fixed)

if nargin > 5
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

if nargin < 3
    stk_error ('Not enough input arguments.', 'NotEnoughInputArgs');
end

%% Change parameters

if nargin > 3 && ~isempty(param_changed)
    if nargin == 5 && ~isempty(param_fixed) && any(~isnan(param_fixed))
        new_param = param_fixed;
        new_param(isnan(param_fixed)) = param_changed;
        % change empty param by the proposition
    else
        new_param = param_changed;
    end
    model = stk_set_optimizable_parameters(model, new_param);
end

%% Compute the log posterior
try
    logpost = -stk_param_mlogpost(model, xi, zi);
catch err
    if strcmp(err.identifier, 'STK:stk_cholcov:InvalidArgument')
        % Problem with the covariance matrix (Inf, NaN, bad conditioning)
        logpost = +Inf;
    else
        rethrow(err);
    end
end

end

