% STK_HAARIO_SUGGEST_C0 suggests a value for C0 in stk_haario_adaptive_metropolis
%
% CALL: C0 = stk_haario_suggest_C0(DISTRIB, BOX_PARAM, X0)
% 
%   proposes an initial covariance C0 for stk_haario_adaptive_metropolis.
%   DISTRIB is the probability distribution, BOX_PARAM defines its
%   supports, X0 is the initial value which will be used in
%   stk_haario_adaptive_metropolis.
%
% CALL: [C0, X0] = stk_haario_suggest_C0(DISTRIB, BOX_PARAM)
%
%   proposes an initial value for X0, as the center of BOX_PARAM.
%
% CALL: C0 = stk_haario_suggest_C0(DISTRIB, BOX_PARAM, X0, LOGDISTRIB)
% CALL: [C0, X0] = stk_haario_suggest_C0(DISTRIB, BOX_PARAM, [], LOGDISTRIB)
%
%   specifies if DISTRIB is the probability distribution (LOGDISTRIB = false),
%   or its logarithm (LOGDISTRIB = true). Improve numerical accuracies.
%   Warning : if LOGDISTRIB is true, DISTRIB must be the logarithm, not its
%   opposite.
%   Default value : false.
%
% NOTE: this function suggests a value for C0, it does not pretend to give
%   any real value of anything. Use carefully.
%
% See also: stk_haario_adaptive_metropolis
function [C0, x0] = stk_haario_suggest_C0(distrib, box_param, x0, logDistrib)

if nargin > 4
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Check arguments
% check required arguments
if nargin < 2 || isempty(distrib) || isempty(box_param)
    stk_error ('You must define the distribution and the parameter box.',...
        'NotEnoughInputArgs');
end

% test box
if ~ isa (box_param, 'stk_hrect')
    box_param = stk_hrect (box_param);%check size(box) = 2 x dim and box(1, :) <= box(2, :)
end
box_param = double(box_param);  %to apply operations on box (division)
dim = size(box_param, 2);

if any(box_param(1, :) == box_param(2, :))
    stk_error('There is at least one dimension where the two bounds of the box are equal.', 'InvalidBox');
end

%x0
if nargin < 3 || isempty(x0)
    x0 = (box_param(1, :) + box_param(2, :))/2; % default value : center of the domain
else
    if ~ isequal (size(x0), [1, dim])
        %check size(x0) = 1 x dim
        if isequal(size(x0), [dim, 1])
            x0 = x0';
        else
            errmsg = 'x0 should be a row, with the same number of columns as box.';
            stk_error (errmsg, 'IncorrectSize');
        end
    end
    if any( (x0(1, :) < box_param(1, :)) + (x0(1, :) > box_param(2, :)) )
        %check that every element of x0 is higher than box(1, :) and lower
        %than box(2, :).
        stk_error ('x0 must be include inside the box.', 'InvalidArgument');
    end
end

% logDistrib
if nargin < 4 || isempty(logDistrib)
    logDistrib = false;	% By default, the function is the real distribution (not its logarithm)
else
    if ~islogical(logDistrib)
        try
            logDistrib = logical(logDistrib);
        catch
            stk_error('logDistrib must be a logical.', 'InvalidArgument');
        end
    end
    logDistrib = logDistrib(1);
end

%% Calculate the range
range0 = NaN(dim, 1);
kste = 10;

nTryBest = 16;
nTryKste = 16;

% We will work dimension by dimension
for kd = 1:dim
    % 1 : find the best value along the axis
    % One dimensional function along the dim
    local_distrib = @(x_kd)(-distrib([x0(1:(kd - 1)), x_kd, x0((kd + 1):dim)]));
    % Minus Distribution, because it is a minimization
    
    % Find bounds of optimization
    x_test = regular_space_points(nTryBest, box_param(1, kd), box_param(2, kd));
    distrib_x_test = NaN(nTryBest, 1);
    for iTry = 1:nTryBest
        distrib_x_test(iTry, 1) = local_distrib(x_test(iTry, 1));
    end
    
    [~, i_min1] = min(distrib_x_test);
    distrib_x_test(i_min1) = +Inf;
    [~, i_min2] = min(distrib_x_test);
    
    x_min1 = x_test(i_min1);
    x_min2 = x_test(i_min2);
    
    % Optimization
    if x_min1 > x_min2
        [x_kd_min, distrib_min] = fminbnd(local_distrib, x_min2, x_min1);
    else % if x_min1 < x_min2
        [x_kd_min, distrib_min] = fminbnd(local_distrib, x_min1, x_min2);
    end
    
    % 2 : find the value of nearest the maximum divided by kste = 10,
    % along each dimension.
    
    if logDistrib
        farFromKste = @(x_kd)( (local_distrib(x_kd) - (distrib_min + log(kste)) ).^2 );
    else
        farFromKste = @(x_kd)( (local_distrib(x_kd) - (distrib_min/kste)).^2 );
    end
    
    % Lower range
    if x_kd_min == box_param(1, kd)
         nearKste_lower = x_kd_min;
    else % standard case
        x_kste_box = regular_space_points(nTryKste, box_param(1, kd), x_kd_min);
        iPar = nTryKste;
        best_distrib = distrib_min;
        ind_best_distrib = [];
        notFound = true;
        
        % Find the first value lower than x_kd_min, whose the distribution
        % is lower than max_distrib/kste (if logDistrib == true, lower
        % than max_distrib - log(kste) ).
        % Warning : we will work with -distrib
        while iPar > 0 && notFound
            current_distrib = local_distrib(x_kste_box(iPar, 1));
            if current_distrib > best_distrib
                best_distrib = current_distrib;
                ind_best_distrib = iPar;
            end
            if logDistrib && best_distrib > (distrib_min + log(kste))
                notFound = false;
            elseif ~logDistrib && best_distrib > (distrib_min/kste)
                notFound = false;
            end
            iPar = iPar - 1;
        end
        
        if ~isempty(ind_best_distrib)
            lowerOpt = x_kste_box(ind_best_distrib);
        else
            lowerOpt = box_param(1, kd);
        end
        % Optimization
        nearKste_lower = fminbnd(farFromKste, lowerOpt, x_kd_min);
    end
    
    % Upper range
    if x_kd_min == box_param(2, kd)
         nearKste_upper = x_kd_min;
    else % standard case
        x_kste_box = regular_space_points(nTryKste, x_kd_min, box_param(2, kd));
        iPar = 1;
        best_distrib = distrib_min;
        ind_best_distrib = [];
        notFound = true;
        
        % idem as before :
        % find the first value upper than x_kd_min, whose the distribution
        % is lower than max_distrib/kste (if logDistrib == true, lower than
        % max_distrib - log(kste) ).
        while (iPar < nTryKste + 1) && notFound
            current_distrib = local_distrib(x_kste_box(iPar, 1));
            if current_distrib > best_distrib
                best_distrib = current_distrib;
                ind_best_distrib = iPar;
            end
            if logDistrib && best_distrib > (distrib_min + log(kste))
                notFound = false;
            elseif ~logDistrib && best_distrib > (distrib_min/kste)
                notFound = false;
            end
            iPar = iPar + 1;
        end
        
        if ~isempty(ind_best_distrib)
            upperOpt = x_kste_box(ind_best_distrib);
        else
            upperOpt = box_param(2, kd);
        end
        
        nearKste_upper = fminbnd(farFromKste, x_kd_min, upperOpt);
    end
    
    % 3 : calculate range
    range0(kd, 1) = (nearKste_upper - nearKste_lower)/2;
end

%% Deduce C0 from range 0
sd = (2.4)/dim;     % Following Haario's advice
C0 = diag( sd*(range0.^2));

end

%% Create a local function to have points regularly spaced in an interval
function x = regular_space_points(n, minBnd, maxBnd)
%dx = (maxBnd - minBnd)/n;
% x = ((minBnd + dx/2):dx:(maxBnd - dx/2))

x = linspace(minBnd, maxBnd, 2*n + 1);
x = x(2:2:(2*n))';

end