% PLOT_DENSITY_PARAMETERS plots the density of sampled parameters
%
% CALL: H_PAR = plot_density_parameters(SIM_PARAM, MODEL, REAL_VALUE, ND, PLOT_PRIOR)
function h_par = plot_density_parameters(sim_param, model, real_value, nd, plot_prior)

dim_param = size(sim_param, 2);

% Type of model
have_reg = type_covariance(model.covariance_type);	% Is the second parameter the regularity?
is_noise = stk_isnoisy(model);                      % Is there noise?

% Computes the real values
if nargin < 3 || isempty(real_value)
    real_value = NaN(dim_param, 1);
else
    real_value = real_value(:);
    if length(real_value) == 1 && is_noise
        % If there is only one real value, assume its the real noise
        % standard deviation?
        real_value = [nan(dim_param - 1, 1); real_value];
    elseif length(real_value) ~= dim_param
        real_value = NaN(dim_param, 1);
    end
end

% Number of points in the plot
if nargin < 4 || isempty(nd)
    nd = 100;
end

% Does it plot prior distribution ?
if nargin < 5 || isempty(plot_prior)
    plot_prior = true;
end

% Plot a kernel density or an histogram
plot_ksdensity = (exist ('OCTAVE_VERSION', 'builtin') ~= 5);
% isoctave <=> (exist ('OCTAVE_VERSION', 'builtin') == 5);

%% Style and colors of the figure
color_prior = [0.0, 0.0, 1];
color_posterior = [1.0, 0.7, 0.0];
color_MaP = [1.0, 0, 0.7];
color_real = [0.0, 0.0, 0.0];

% Prior density
style_prior = {'LineStyle', '--', 'Color', color_prior, 'LineWidth', 2};
if plot_ksdensity
    % Posterior density
    style_dens = {'LineStyle', '-', 'Color', color_posterior, 'LineWidth', 3};
    
    % Density of the model
    style_model = {'LineStyle', 'none', 'Marker', 'o', 'MarkerSize', 6,...
        'Color', color_MaP, 'MarkerFaceColor', color_MaP};

else % if plot histogram    
    % Posterior histogram
    style_hist = {'LineStyle', '-', 'LineWidth', 0.5, 'FaceColor', color_posterior,...
        'EdgeColor', 'k', 'BarWidth', 1};
    
    % Density of the model
    style_model = {'LineStyle', '--', 'Marker', 'none', 'LineWidth', 1,...
        'Color', color_MaP};
end
% Real value of (a) parameter(s)
style_real = {'LineStyle', '-.', 'Marker', 'none', 'Color', color_real, 'LineWidth', 1};


%% Plot the kernel density of parameters

h_par_hidden = stk_figure('Density of sampled parameters');

% For each parameters
for jd_par = 1:dim_param
    % Get the j-th simulated parameters
    sim_par_j = sim_param(:, jd_par);
    
    % Real value (NaN = not defined)
    real_val_j = real_value(jd_par, 1);
    % "can" for canonical parameters
    mu_prior = nan;
    sigma_prior = nan;
    
    if jd_par == 1
        % logsig2
        name_opt = 'log(\sigma^2)';
        name_can = '\sigma';
        
        sim_par_can = exp(0.5*sim_par_j);
        
        par_opt = model.param(jd_par);
        par_can = exp(0.5*par_opt);
        
        real_par_j = 2*log(real_val_j);
        
        if plot_prior && isfield(model, 'prior')
            mu_prior = model.prior.mean(jd_par);
            sigma_prior = 1./sqrt(model.prior.invcov(jd_par, jd_par));
            
            ktrans = 2;
            % ginv = @(x)(2*log(x));
            % dginv_dp = @(x)(2./x);
        end
        
    elseif jd_par == dim_param && is_noise
        % log-noise variance (only if there is noise)
        name_opt = 'log(\lambda)';
        name_can = '\lambda^{0.5}';
        
        sim_par_can = exp(0.5*sim_par_j);
        
        par_opt = model.lognoisevariance;
        par_can = exp(0.5*par_opt);
        
        % Real value of the standard deviation of the noise
        real_par_j = 2*log(real_val_j);
        
        if plot_prior && isfield(model, 'noiseprior')
            mu_prior = model.noiseprior.mean;
            sigma_prior = sqrt(model.noiseprior.var);
            
            ktrans = 2;
            % ginv = @(x)(2*log(x));
            % dginv_dp = @(x)(2./x);
        end
        
    elseif jd_par == 2 && have_reg
        % logreg (only if the parameter is optimized)
        name_opt = 'log(\nu)';
        name_can = '\nu';
        
        sim_par_can = exp(sim_par_j);
        
        par_opt = model.param(jd_par);
        par_can = exp(par_opt);
        
        real_par_j = log(real_val_j);
        
        if plot_prior && isfield(model, 'prior')
            mu_prior = model.prior.mean(jd_par);
            sigma_prior = 1./sqrt(model.prior.invcov(jd_par, jd_par));
            
            ktrans = 1;
            % ginv = @log;
            % dginv_dp = @(x)(1./x);
        end
        
    else % otherwise is a log-accuracy
        % logacc
        name_opt = 'log(1/\rho)';
        name_can = '\rho';
        
        sim_par_can = exp(-sim_par_j);
        
        par_opt = model.param(jd_par);
        par_can = exp(-par_opt);
        
        real_par_j = -log(real_val_j);
        
        if plot_prior && isfield(model, 'prior')
            mu_prior = model.prior.mean(jd_par);
            sigma_prior = 1./sqrt(model.prior.invcov(jd_par, jd_par));
            
            ktrans = -1;
            % ginv = @(x)(-log(x));
            % dginv_dp = @(x)(1./x);
        end
    end
    
    % Fix limits
    lim_ax_opt = [min(sim_par_j), max(max(sim_par_j), real_par_j)];
    lim_ax_can = [0, max(max(sim_par_can), real_val_j)];
    
    % Set axis
    ax_sim_opt = linspace(lim_ax_opt(1), lim_ax_opt(2),   nd);
    ax_sim_can = linspace(lim_ax_can(1), lim_ax_can(2), 2*nd);
    
    
    if plot_ksdensity
        % Kernel density regression
        ks_sim_opt = ksdensity(sim_par_j, ax_sim_opt, 'support', 'unbounded');
        ks_sim_can = ksdensity(sim_par_can, ax_sim_can, 'support', 'positive');
        
        % Parameters with maximize the posterior distribution
        ks_model_opt = ksdensity(sim_par_j, par_opt, 'support', 'unbounded');
        ks_model_can = ksdensity(sim_par_can, par_can, 'support', 'positive');
        
        % Plot density in the space of optimized parameters
        y_sub1 = stk_subplot(2, dim_param, jd_par);
        hold off;
        % Density
        plot(ax_sim_opt, ks_sim_opt, style_dens{:});
        hold on;
        % Optimal parameters
        plot(par_opt, ks_model_opt, style_model{:});
        % Prior distribution
        if ~isnan(mu_prior)
            plot(ax_sim_opt, normpdf(ax_sim_opt, mu_prior, sigma_prior), style_prior{:});
        end
        % Real value
        if ~isnan(real_par_j)
            plot(real_par_j*ones(1, 2), get(y_sub1, 'Ylim'), style_real{:});
        end
        xlim(lim_ax_opt);
        stk_xlabel(name_opt);
        
        % Plot density in the space of canonical parameters
        y_sub2 = stk_subplot(2, dim_param, dim_param + jd_par);
        hold off;
        % Density
        plot(ax_sim_can, ks_sim_can, style_dens{:});
        % Optimal parameters
        hold on;
        plot(par_can, ks_model_can, style_model{:});
        % Prior distribution
        if ~isnan(mu_prior)
            plot(ax_sim_can, (abs(ktrans)./ax_sim_can).*normpdf(ktrans*log(ax_sim_can),...
                mu_prior, sigma_prior), style_prior{:});
        end
        % Real value
        if ~isnan(real_val_j)
            plot(real_val_j*ones(1, 2), get(y_sub2, 'Ylim'), style_real{:});
        end
        xlim(lim_ax_can);
        stk_xlabel(name_can);
        
    else
        % histogram
        nb_param = size(sim_par_j, 1);
        n_class = ceil(2 * (nb_param^(1/3)));
        [hc_opt, xc_opt] = hist(sim_par_j, n_class);
        dx_opt = mean(diff(xc_opt));
        hc_opt = hc_opt/(dx_opt*nb_param);
        
        [hc_can, xc_can] = hist(sim_par_can, n_class);
        dx_can = mean(diff(xc_can));
        hc_can = hc_can/(dx_can*nb_param);
        
        % Plot histogram in the space of optimized parameters
        y_sub1 = stk_subplot(2, dim_param, jd_par);
        hold off;
        % Histogram
        bar(xc_opt, hc_opt, style_hist{:});
        hold on;
        % Prior distribution
        if ~isnan(mu_prior)
            plot(ax_sim_opt, normpdf(ax_sim_opt, mu_prior, sigma_prior), style_prior{:});
        end
        % Optimal parameters
        plot(par_opt*ones(1, 2), get(y_sub1, 'Ylim'), style_model{:});
        % Real value
        if ~isnan(real_par_j)
            plot(real_par_j*ones(1, 2), get(y_sub1, 'Ylim'), style_real{:});
        end
        xlim(lim_ax_opt);
        stk_xlabel(name_opt);
        
        % Plot density in the space of canonical parameters
        y_sub2 = stk_subplot(2, dim_param, dim_param + jd_par);
        hold off;
        % Histogram
        bar(xc_can, hc_can, style_hist{:});
        hold on;
        % Prior distribution
        if ~isnan(mu_prior)
            plot(ax_sim_can, (abs(ktrans)./ax_sim_can).*normpdf(ktrans*log(ax_sim_can),...
                mu_prior, sigma_prior), style_prior{:});
        end
        % Optimal parameters
        plot(par_can*ones(2, 1), get(y_sub2, 'Ylim'), style_model{:});
        % Real value
        if ~isnan(real_val_j)
            plot(real_val_j*ones(1, 2), get(y_sub2, 'Ylim'), style_real{:});
        end
        xlim(lim_ax_can);
        stk_xlabel(name_can);
    end
    
end

% Return the handle if asked
if nargout > 0
    h_par = h_par_hidden;
end

end

