% SAMPLING_SMC proposes an example of script to sample parameters
%
% CALL: [sim_param, weights] = sampling_smc(nb_param, model, xi, zi, nthres, t0)
function [sim_param, weights] = sampling_smc(nb_param, model, xi, zi, nthres, t0)


%% Default values
if nargin < 5 || isempty(nthres)
    nthres = floor(0.9*nb_param); % Threshold of effective number
end

if nargin < 6 || isempty(t0)
    t0 = 10;   % Default length of the chains
end

ni = size(xi, 1);

%% Bounds of parameters
[lb_cov, ub_cov] = stk_param_getdefaultbounds(model.covariance_type, model.param, xi, zi);
if stk_isnoisy(model) && isscalar(model.lognoisevariance)
    lnv_init = model.lognoisevariance;
    
    lb_lnv = min(log(eps)    , lnv_init  - 0.5);
    ub_lnv = max(log(var(zi)), lnv_init) + 0.5;
    
else
    lb_lnv = [];
    ub_lnv = [];
end

bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];

%% Sampling
% i = 1: initialization
% Samples parameters according to the prior distribution
mu_param = model.prior.mean;
if isequal( diag(diag(model.prior.invcov)), model.prior.invcov )
    % if is a diagonal matrix
    sigma2_param = diag(1./diag(model.prior.invcov));
else
    sigma2_param = inv(model.prior.invcov);
end

if stk_isnoisy(model) && isscalar(model.lognoisevariance)
    mu_param = [mu_param; model.noiseprior.mean];
    sigma2_param = [sigma2_param, zeros(size(sigma2_param, 1), 1);
        zeros(1, size(sigma2_param, 2)), model.noiseprior.var;];
end
% Number of parameters
dim_param = length(mu_param);

% sampling according prior distribution
if isequal( diag(diag(sigma2_param)), sigma2_param )
    % if isdiag
    sigma_param = diag(sqrt(diag(sigma2_param)));
else
    sigma_param = chol(sigma2_param);   % Cholesky decomposition
end
sim_param = repmat(mu_param', nb_param, 1) + randn(nb_param, dim_param)*sigma_param;
weights = ones(nb_param, 1)/nb_param;

% Compute initial Anti-restricted likelihood
relik = NaN(nb_param, 1);
for lrp = 1:nb_param
    relik(lrp, 1) = stk_param_relik_val(model, xi(1, :), zi(1, :), sim_param(lrp, :));
end

for i = 2:ni
    % SMC - 1: update weights
    [weights, relik] = stk_smc_weights(sim_param, model, xi(1:(i - 1), :),...
        zi(1:(i - 1), :), xi(i, :), zi(i, :), weights, relik);
    % SMC - 1bis: compute effective numbers
    Neff = stk_smc_effect(weights);
    
    display(sprintf('Loop %d/%d. Effective number = %.1f', i, ni, Neff));
    if Neff < nthres % if effective numbers is lower than the threshold
        % SMC - 2: re-sample
        [sim_param, weights] = stk_smc_sample(sim_param, weights);
        
        % Delete old computations of Anti-restricted likelihood
        relik = [];
        
        % Update posterior distribution
        logPost_i = @(p)(-stk_param_relik_val(model, xi(1:i, :), zi(1:i, :), p));
        % Warning: -stk_param_relik, because it is the
		% anti-restricted likelihood which is computed!
        
        % Compute covariance of parameters
        t0_loop = t0;    % Default length of the chains
        cov_param = stk_haario_suggest_C0(logPost_i, bnd_param, [], true);
        
        % SMC - 3: move the parameters
        sim_param = stk_smc_move(sim_param, logPost_i, t0_loop, cov_param);
    end
    
end

end

