% STK_PARAM_RELIK_VAL computes the Anti-Relik after changing the parameters.
%
% CALL: [ARL, dARL_dtheta, dARL_dLNV] = stk_param_relik (MODEL, XI, YI, PARAM_VALUES)
%
%   computes stk_param_relik after having set the parameters of MODEL
%   with PARAM_VALUES. PARAM_VALUES must be a vector, whose its length
%   must be equals to the number of covariance parameters, plus 1, if MODEL
%   is noisy with a homoscedastic noise; 0 otherwise.
%
%   The function returns -Inf, if the parameters cannot fit the model.
%
% See also: stk_param_relik
function [rl, drl_param, drl_lnv] = stk_param_relik_val (model, xi, yi, param_vals)

if nargin > 4
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

%% Change parameters
if nargin > 3 && ~isempty(param_vals)
    % Number of covariance parameters (assume there is always
    % covariance parameters to optimize)
    nb_param_cov = length(model.param(:));
    
    if stk_isnoisy(model) && isscalar(model.lognoisevariance)
        nb_param_lnv = 1;   % Do not consider heteroscedastic case
        
        % Check length of param_vals
        if numel(param_vals) ~= (nb_param_cov + nb_param_lnv)
            stk_error(['The length of the param_vals must fit the number of',...
                ' parameters.'], 'IncorrectSize');
        end
        
        % Change covariance parameter
        model.param(:) = param_vals(1:nb_param_cov);
        % Change noise variance parameter
        model.lognoisevariance(:) =  param_vals(nb_param_cov + (1:nb_param_lnv));
    else
        % nb_param_lnv = 0
        % Check length of param_vals
        if numel(param_vals) ~= nb_param_cov
            stk_error(['The length of the param_vals must fit the number of',...
                ' parameters.'], 'IncorrectSize');
        end
        
        % Change covariance parameter values
        model.param(:) = param_vals(:);
    end
end

%% Compute the Restricted Likelihood

try
    
    switch nargout
        case {0, 1}
            rl = stk_param_relik (model, xi, yi);
        case 2
            [rl, drl_param] = stk_param_relik (model, xi, yi);
        case 3
            [rl, drl_param, drl_lnv] = stk_param_relik (model, xi, yi);
        otherwise
            stk_error ('Too many output arguments.', 'TooManyOutputArgs');
    end
    
catch
    err = lasterror();
    if strcmp(err.identifier, 'STK:stk_cholcov:InvalidArgument')
        % Problem with the covariance matrix (Inf, NaN, bad conditioning)
        rl = -Inf;
        drl_param = NaN(length(model.param(:)), 1);
        drl_lnv = NaN(length(model.lognoisevariance(:)), 1);
    elseif (exist ('OCTAVE_VERSION', 'builtin') == 5)...
        && strcmp(err.identifier, '') && strcmp(err.stack(1).name, 'stk_cholcov')
        % Error in stk_cholcov, when ask empty matrix (only true on octave)
        % C = stk_cholcov([]) <=> [C, p] = chol([])
        
        % isoctave <=> (exist ('OCTAVE_VERSION', 'builtin') == 5);
        
        % Return a prior
        PARAMPRIOR = isfield (model, 'prior');
        NOISEPRIOR = isfield (model, 'noiseprior');
        
        % Parameters of the model
        param = model.param(:);
        lnv = model.lognoisevariance(:);
        
        % Future results
        rl = 0;
        drl_param = zeros(length(param), 1);
        drl_lnv   = zeros(length(lnv), 1);
        
        % Prior on parameters
        if PARAMPRIOR
            % param - mean
            delta_p = param - model.prior.mean;
            % Compute derivative
            drl_param = model.prior.invcov * delta_p;
            % Compute -log(prior)
            rl = rl + 0.5 * delta_p' * drl_param;
            
        end
        
        % Prior on noise variance
        if NOISEPRIOR
            % param - mean
            delta_lnv = lnv - model.noiseprior.mean;
            % Compute derivative
            if isfield (model.noiseprior, 'invcov')
                drl_lnv = model.noiseprior.invcov * delta_lnv;
            else % assume isfield (model.noiseprior, 'var')
                drl_lnv = (model.noiseprior.var\delta_lnv);
            end
            % Compute -log(prior)
            rl = rl + 0.5 * (delta_lnv' * drl_lnv);
        end
    else
        rethrow(err);
    end
end

end

