% STK_SMC_PREDICT performs a posterior prediction from data and
% simulated parameters
%
% CALL: ZP = stk_smc_predict(SIM_PARAM, WEIGHTS, MODEL, XI, ZI, XP)
%
%   returns the posterior mean and variance at XP, of the process described
%   by the model, and with observations (XI, ZI). SIM_PARAM are nb_param
%   samples of the hyper-parameters, weighted by weights.
%
% See also: stk_predict
function zp = stk_smc_predict(sim_param, weights, model, xi, zi, xp)


% Number of prediction points
np = size(xp, 1);
% Number of parameters
nb_param = size(sim_param, 1);

% Weights
if isempty(weights)
    weights = ones(nb_param, 1)/nb_param;
else
    weights = weights(:);
    % Assert normalization
    weights = weights/sum(weights);
end
    
is_noise = stk_isnoisy(model);

%% Predictions taking into account on simulated parameters
zp_mean  = zeros(np, 1);   % E(E[xi(x)|theta]) = E(E[Z|theta])
zp2_mean = zeros(np, 1);   % E(E[xi(x)^2|theta])
if is_noise
    zn2_mean = zeros(np, 1);   % E(E[Z^2|theta])
end

for kpar = 1:nb_param
    % For each sampled parameters
    
    % 1: change the parameters on the model
    if is_noise && isscalar(model.lognoisevariance);
        % Assume nb_param_lnv = 1; Do not consider heteroscedastic case
        
        % Change covariance parameter
        model.param(:) = sim_param(kpar, 1:(end - 1));
        % Change noise variance parameter
        model.lognoisevariance(:) = sim_param(kpar, end);
    else
        % nb_param_lnv = 0
        % Change covariance parameter values
        model.param(:) = sim_param(kpar, :);
    end
    
    % 2: Predict knowing the parameters
    zp_k = stk_predict(model, xi, zi, xp);
    zp_k_mean = zp_k.mean;
    zp2_k_mean = zp_k.var + zp_k.mean.^2;
    
    if is_noise
        zn_var_k = zp_k.var + exp(model.lognoisevariance);
        zn2_k_mean = zn_var_k + zp_k.mean.^2;
        
        % 3: Update the first and second moment
        zn2_mean = zn2_mean + weights(kpar, 1)*zn2_k_mean;
    end
    
    zp_mean  =  zp_mean + weights(kpar, 1)*zp_k_mean;
    zp2_mean = zp2_mean + weights(kpar, 1)*zp2_k_mean;
    
end

% Concatenate results on one data-frame
zp = stk_dataframe([zp_mean, zp2_mean - zp_mean.^2], {'mean', 'var'});
% E[xi(x)] = E[Z] = E(E[Z|theta])
% Var[xi(x)] = E[xi(x)^2] - E[xi(x)]^2 = E(E[xi(x)^2|theta]) - E(E[xi(x)|theta]^2)

if is_noise
    zn = stk_dataframe(zn2_mean - zp_mean.^2, {'var_obs'});
    % Var[Z] = E[Z^2] - E[Z]^2 = E(E[Z^2|theta]) - E(E[Z|theta]^2)
    zp = [zp, zn];
end

end

