% STK_UPDATE_RECURSIVE computes the new mean and new variance using
% recursive formulas
%
% CALL : [M_n, V_n] = stk_update_recursive(N, M_nM1, V_nM1, X_n)
%
%   computes the mean M_n = mean([X_1, ...,X_nM1 , X_n]) and the variance
%   V_n = var([X_1, ...,X_nM1 , X_n], 0), using the number N,
%   the previous variance V_nM1 = var([X_1, ..., X_nM1], 0), the previous
%   mean M_nM1 = mean([X_1, ..., X_nM1]), and the new value X_n.
%
% EXAMPLE:
%
%   n = 4;
%   len = randi([1, 10], n, 1);
%   values = cell(n, 1);
%   mean_prev = NaN(n, 1);
%   var_prev = NaN(n, 1);
% 
%   last = NaN(n, 1);
% 
%   mean_res = NaN(n, 1);
%   var_res = NaN(n, 1);
%   for k = 1:n
%       values{k} = rand(len(k), 1);
%       mean_prev(k) = mean(values{k}(1:(end - 1), 1));
%       var_prev(k)  = var(values{k}(1:(end - 1), 1));
%       last(k) = values{k}(end);
%       mean_res(k) = mean(values{k});
%       var_res(k)  = var(values{k});
%   end
% 
%   [m, v] = stk_update_recursive(len, var_prev, mean_prev, last);
%   v(len == 1) = 0;
%   stk_isequal_tolabs(m, mean_res);
%   stk_isequal_tolabs(v, var_res);
%
% See also: stk_mean_recursive, stk_var_recursive
function [m_n, v_n] = stk_update_recursive(n, m_nM1, v_nM1, x_n)


if nargin ~= 4
    if nargin < 4
        stk_error('Not enough input arguments', 'NotEnoughInputArgs');
    else
        stk_error('Too many input arguments', 'TooManyInputArgs');
    end
end

%% Update
v_n = stk_var_recursive (n, v_nM1, m_nM1, x_n);
m_n = stk_mean_recursive(n,        m_nM1, x_n);

end

