function k = stk_brownian_cov(param, t, s, diff, pairwise)
% k = stk_brownian_cov(param, t, s, diff, pairwise)
%
% k = s^2*min{t, s}^L

if nargin > 5
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%%
persistent t0 s0 pairwise0 L0 I0
persistent dk logminMat

%%

% process input arguments
t = double (t);
s = double (s);
if nargin < 5
    pairwise = false;
    if nargin < 4
        diff = -1;
    end
end


% check size consistency
dim = size (t, 2);
if dim ~= 1
    stk_error('ti must be a vector of dimension 1.', 'InvalidArgument');
end

if (size (s, 2) ~= dim),
    stk_error ('ti and si have incompatible sizes.', 'InvalidArgument');
end

if numel(param) < 2
    stk_error('Param must have length upper than 2.', 'InvalidArgument');
elseif numel(param) > 3
    stk_error('Param must have length lower than 3.', 'InvalidArgument');
end

Sigma2 = exp (param(1));
L      = exp (param(2));
if numel(param) == 3
    I = param(3);
else % numel(param) = 2
    I = 0;
end

% check parameter values
if Sigma2 <= 0
    error ('Incorrect parameter value.');
end


if (diff ~= -1) && ((diff < 1) || (diff > 3))
    stk_error ('Incorrect value for the ''diff'' parameter.', 'InvalidArgument');
end

%%

if isempty(s0) || isempty(t0) || isempty(pairwise0) || isempty(L0) || isempty(I0)...
        || ~isequal(t, t0) || ~isequal(s, s0) || ~isequal(pairwise, pairwise0)...
        || ~isequal(L, L0) || ~isequal(I, I0)
    
    % Compute |t - I| and |s - I|
    tI = abs(t - I);
    sI = abs(s - I);
    if pairwise
        % Compute min(tI, sI)^L
        minMatrix = min(tI, sI);
        dk = minMatrix.^L;
        logminMat = log(minMatrix);
    else
        % Compute min(tI^L, sI^L)
        tL = tI.^L;
        sL = sI.^L;
        dk = min(repmat(tL, 1, length(s)), repmat(sL', length(t), 1));
        logminMat = log(min(repmat(t, 1, length(s)), repmat(s', length(t), 1)));
    end
    
    s0 = s;
    t0 = t;
    pairwise0 = pairwise;
    L0 = L;
    I0 = I;
end

%%
switch diff
    case -1
        % Return the covariance (no derivation)
        k = Sigma2*dk;
    case 1
        % Compute derivative according to log(sig2)
        k = Sigma2*dk;
    case 2
        % Compute derivative according to log(L)
        k = L*Sigma2*(logminMat.*dk);
    case 3
        % Compute derivative according to I (no derivative)
        k = zeros(size(dk));
    otherwise
        stk_error ('Incorrect value for the ''diff'' parameter.', 'InvalidArgument');
end

end

