% CALL : noimo_mllnv = stk_noisemodel_multilev(levels)
%
% CALL : noimo_mllnv = stk_noisemodel_multilev(levels, value)
%
% CALL : noimo_mllnv = stk_noisemodel_multilev(levels, value, isLog)
%
% CALL : noimo_mllnv = stk_noisemodel_multilev(levels, value, isLog, lambda0, var_lev, var_mean)
%
% Create a noise model with a multi-level noise variance.
%
% - level : the different levels;
% - value (default value : NaN(length(levels), 1)) : noise variance;
% - isLog (default value : false) : indicate if the provided value is the
% log-variance (true) or the variance (false).
%
% - lambda0, var_lev, var_mean defines a gaussian prior, with mean lambda0,
% and covariance var_lev*eye(nb_lev) + var_mean*ones(nb_lev).
% If there are not provided, or if var_lev = +Inf, no prior is added.
%
% Default values are : lambda0 = zeros(nb_lev, 1), var_lev = 0.25, and
% var_mean = +Inf
function noimo_mllnv = stk_noisemodel_multilev(levels, value, isLog, lambda0, var_lev, var_mean)

if nargin > 6
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Default values for parameters
if nargin < 1
    levels = [1; 2];
elseif isempty(levels)
    if nargin > 1 && ~isempty(value)
        levels = (1:numel(value))';
    else
        levels = [1; 2];
    end
else % if levels is provided
    levels = levels(:);                 % assert vector
    levels = unique(levels, 'stable');	% assert uniqueness
end

nb_lev = length(levels);

if nargin < 2 || isempty(value)
    logvar = NaN(nb_lev, 1);   % default value for log-variance
else
    if nargin < 3 || isempty(isLog) || ~isLog
        % default value for isLog : false
        if value < 0
            stk_error('Noise variance must be a positive parameter.', 'InvalidArgument');
        end
        logvar = log(value);
    else
        logvar = value;
    end
    
    logvar = logvar(:); % assert columns
    if length(logvar) ~= nb_lev
        stk_error('Noise variance and levels does not have the same size.', 'IncorrectSize');
    end
end

%% function
func = 'stk_multilevnoisecov_out';

%% Parameters
param = stk_param_multilevel_lnv(logvar, levels);

%% Prior
if nargin < 4 || (nargin > 4 && ~isempty(var_lev) && isinf(var_lev))
    prior = [];
else
    % default values
    % Lambda0
    if isempty(lambda0)
        lambda0 = zeros(nb_lev, 1);
    else
        lambda0 = lambda0(:);   % assert vector
        if any(isnan(lambda0)) || any(isinf(lambda0))
            stk_error('Lambda0 contains NaN or Inf value.', 'matrixWithNaNorInf');
        end
        
        if length(lambda0) == 1
            lambda0 = lambda0*ones(nb_lev, 1);
        elseif length(lambda0) ~= nb_lev
            stk_error(['Lambda0 must be a scalar or a vector with the same',...
                ' size as level.'], 'IncorrectSize');
        end % elseif length(lambda0) == nb_lev, this is good
    end
    
    % Diagonal of the covariance matrix
    if nargin < 5 || isempty(var_lev)
        var_lev = 0.5^2;
    else
        if ~isscalar(var_lev) || isnan(var_lev)
            stk_error('var_lev must be a non-nan scalar value.', 'InvalidArgument');
        end
        
        if var_lev < 0
            stk_error('var_lev is a variance and must be positive.', 'InvalidArgument');
        end
    end
    
    % Complete term of the covariance matrix
    if nargin < 6 || isempty(var_mean)
        var_mean = +Inf;
    else
        if ~isscalar(var_mean) || isnan(var_mean)
            stk_error('var_mean must be a non-nan scalar value.', 'InvalidArgument');
        end
        
        if var_mean < 0
            stk_error('var_mean is a variance and must be positive.', 'InvalidArgument');
        end
    end
    
    % Build Gaussian prior
    mean_prior = [lambda0; levels];
    if ~isinf(var_mean)
        covar_prior = var_lev*eye(nb_lev) + var_mean*ones(nb_lev);
    else
        covar_prior = var_lev*(eye(nb_lev) + 100*ones(nb_lev));
    end
    [eigenvects, not_use_vals] = eig(covar_prior);
    % not_use_vals are not used (because I prefer to use my formula, but I
    % asked for it, in order to get the eigenvectors). Ideally, I should
    % prepare myself the eigenvectors, but...
    eigenvals = [var_lev*ones(nb_lev - 1, 1); var_lev + nb_lev*var_mean];
    eigenvals = [eigenvals; zeros(nb_lev, 1)];
    
    eigenvects = [eigenvects, zeros(nb_lev);
        zeros(nb_lev), eye(nb_lev)];
    
    prior = stk_prior_gauss(mean_prior, eigenvals, eigenvects);
end

%% Create the covariance model.
noimo_mllnv = stk_noisemodel(func, param, prior);
noimo_mllnv = class(struct(), 'stk_noisemodel_multilev', noimo_mllnv);


end

