% CALL : noimo_mllnv0 = stk_param_init_lnv_ (noimo_mllnv, model, xi, zi)
% [overload stk_protomodel function]
%
% Suggest an initial value for noimo_mllnv. noimo_mllnv is supposed to
% be the noise model of model.
%
% xi and zi are respectivelly inputs and outputs of the function. The last
% colum of xi gives the level of the function.

function noimo_mllnv0 = stk_param_init_lnv_ (noimo_mllnv, model, xi, zi)

%% Check number of inputs
if nargin > 4,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

if nargin < 4
    stk_error('Too few input arguments.', 'TooFewInputArgs');
end

%% Check size
if size(zi, 1) ~= size(xi, 1)
    stk_error('zi should have the same number of rows as xi.', 'IncorrectSize');
end

if size(zi, 2) ~= 1
    stk_error ('zi should be a column.', 'IncorrectSize');
end

%% Warn about special case: constant response
if (std (double (zi)) == 0)
    warning ('STK:stk_param_init_lnv_:ConstantResponse', ['Constant- ' ...
        'response data: the output of stk_param_estim_lnv is likely ' ...
        'to be unreliable.']);
end

%% Make sure that lognoisevariance is -inf for noiseless models
if ~stk_isnoisy (model)
    model.lognoisevariance = -inf;
    % Sound strange, because you can arrive here only if
    % model.lognoisevariance is a stk_noisemodel_multilev, so if model is
    % noisy.
end

%% Get the value of log_sigma2, which will give the magnitude of lognoisevar
log_sigma2 = stk_get_log_sigma2(model.param);
% if the returned value of log_sigma2 is nan, there is 2 possibilities:
% - either, there are not any field named 'logsig2' in
% model.param.param;
% - or, the value of 'logsig2' is NaN.
if isnan(log_sigma2) && all(~isnan(stk_get_optimizable_parameters(model.param)))
    % if the covariance can be estimated
    kov = feval (model.covariance_type, model.param, xi, xi, -1, true);
    log_sigma2 = mean( log(kov) );
    % return the mean value of log(k(x, x))
end

if isnan(log_sigma2) || isinf(log_sigma2)
    % If log_sigma2 is still not usable (case NaN, or -Inf)
    log_sigma2 = log(var(zi));  % return the log-variance of output
end

%% Furnish a default inital value of the noise variance
noimo_mllnv0 = noimo_mllnv;

mllnv = get(noimo_mllnv, 'param');
nb_lev = length(mllnv.levels);

%% Propose a list of potential values
log_lnv_min = log_sigma2 - 15;  % exp(-15) = 3e-7 approx  -> very small noise
log_lnv_max = log_sigma2 + 15;  % exp(+15) = 3e+7 approx  -> very large noise

nb_test = 8;
% Default : test 8 values for the log-variance
log_lnv_list = linspace(log_lnv_min, log_lnv_max, nb_test)';
log_lnv_list = repmat(log_lnv_list, 1, nb_lev);

log_lnv_list = [log_lnv_list, repmat(mllnv.levels', nb_test, 1)];
    
prior = get(noimo_mllnv, 'prior');
if ~isempty(prior) && isa(prior, 'stk_prior')
    log_lnv_list = stk_param_init_adapt_to_prior(prior, log_lnv_list);
end

nb_test = size(log_lnv_list, 1);
if nb_test > 30
    % extract some default propositions to have only 30 values;
    ind_extract = round(linspace(1, nb_test, 30));
    log_lnv_list = log_lnv_list(ind_extract, :);
end
 
%% Estimating : Try values, return the best
% Initialize parameter search
log_lnv_best = NaN(nb_lev, 1);
aLP_best = +inf;

% Try all values from log_lnv_list
for k_list = 1:size(log_lnv_list, 1)
    mllnv.lognoivar = log_lnv_list(k_list, 1:nb_lev)';
    noimo_mllnv = set(noimo_mllnv, 'param', mllnv);
    
    model.lognoisevariance = noimo_mllnv; % fix the noise model as an
    % multi-level heteroscedastic noise model
    aLP = stk_param_mlogpost (model, xi, zi);   % calculate ante-log posterior
    if (~ isnan (aLP)) && (aLP < aLP_best)
        log_lnv_best = mllnv.lognoivar;
        aLP_best     = aLP;
    end
end

if isinf (aLP_best)
    errmsg = 'Couldn''t find reasonable parameter values... ?!?';
    stk_error (errmsg, 'AlgorithmFailure');
end

%% Build the initial value
mllnv.lognoivar = log_lnv_best;
noimo_mllnv0 = set(noimo_mllnv0, 'param', mllnv);
end

