% CALL : covmo_matiso0 = stk_param_init(covmo_matiso, model, xi, zi, box)
%
% CALL : [covmo_matiso0, noimo0] = stk_param_init(covmo_matiso, model,
%                                       xi, zi, box)
%
% CALL : [covmo_matiso0, noimo0] = stk_param_init(covmo_matiso, model,
%                                       xi, zi, box, do_estim_lnv)
%
% CALL : [covmo_matiso0, noimo0] = stk_param_init(covmo_matiso, model,
%                                       xi, zi, box, do_estim_lnv, varargin)
%
% [overload stk_protomodel function]
%
% Suggest an initial value for covmo_matiso. covmo_matiso is supposed to
% be the covariance model of model.
%
% - xi and zi are respectivelly inputs and outputs of the function.
% - box is the box bounds of xi.
% - varargin is (if provided) the other parameter to use in stk_param_init
% to provide an initial value for lnv (if the model is noised)
% noimo0 is a suggested initial value for model.lognoisevariance (if model
% is noisy, or if do_estim_lnv == true).

function [covmo_matiso0, noimo0] = stk_materncov_iso_param_init ...
    (model, xi, zi, box, do_estim_lnv)

% We expect a particular type of protomodel in model.param
covmo_matiso = model.param;
assert (isa (covmo_matiso, 'stk_covmodel_materniso'));


%% default value for arguments 'box'
if nargin < 5
    box = [];
end

if ~ isa (box, 'stk_hrect')
    if isempty (box)
        box = stk_boundingbox (xi);  % Default: bounding box
    else
        box = stk_hrect (box);
    end
end

%% Checking zi
if ~ isequal (size (zi), [size(xi, 1) 1])
    errmsg = 'zi should be a column, with the same number of rows as xi.';
    stk_error (errmsg, 'IncorrectSize');
end
d = size (xi, 2);

%% Ensure model
% Make sure that lognoisevariance is -inf for noiseless models
if ~stk_isnoisy (model)
    if (nargin < 6) || (~do_estim_lnv)
        % Assume a noiseless model
        model.lognoisevariance = - inf;
    else % do_estim_lnv is provided, and is equal to true
        % Assume a noisy model with constant but unknown noise variance
        model.lognoisevariance = nan;
    end
end

% Ensure backward compatiblity with respect to model.order / model.lm
model = stk_model_fixlm (model);

%% Do estim log-variance ?

noimo0 = model.lognoisevariance;
if (isnumeric (noimo0))  % Backward compatibility
    
    if isscalar(noimo0)
        if nargin < 6  % do_estim_lnv not provided
            do_estim_lnv = isnan (noimo0);
        elseif (~ do_estim_lnv) && isnan (noimo0)
            stk_error (sprintf (['do_estim_lnv is false, but the log-noise ' ...
                'variance is equal to NaN. If you don''t want the log-noise ' ...
                'variance to be estimated, you must provide ' ...
                'values for them!']), 'MissingParameterValue');
        end
        
    else % isvector(lnv)
        % Old-style support for the heteroscedastic case: a numeric vector of
        % log-noise variances was stored in model.lognoisevariance (implicitely
        % associated with the locations xi that would be passed later to
        % stk_noisecov...).
        
        % Noise variance estimation not supported in this case
        if ((nargin >= 6) && do_estim_lnv) || (any (isnan (noimo0)))
            % FIXME: warn that a more modern way of dealing with the heteroscedastic
            % case is now available in STK
            warning('STK:stk_param_init:newfeatures', ['If you want to ', ...
                'define an heteroscedastic noise with parameters to estimate ',...
                'you should create a stk_noisemodel object.']);
            stk_error (['model.lognoisevariance is non-scalar and contains ' ...
                'nans. Noise variance estimation is not supported in the ' ...
                'heteroscedastic case '], 'InvalidArgument');
        else
            do_estim_lnv = false;
        end
    end
    
else  % Case where model.lognoisevariance is a stk_noisemodel
    
    % Estimation of noise variance ?
    %  * if do_estim_lnv is provided, use it to decide
    %  * if not, estimation occurs when lnv parameters contains NaNs
    lnv_vect = stk_get_optimizable_parameters(noimo0);
    if nargin < 6  % do_estim_lnv not provided
        do_estim_lnv = any (isnan (lnv_vect));
    elseif (~ do_estim_lnv) && (any (isnan (lnv_vect)))
        stk_error (sprintf (['do_estim_lnv is false, but some parameter ' ...
            'values are equal to NaN. If you don''t want the parameters of ' ...
            'the noise variance model to be estimated, you must provide ' ...
            'values for them!']), 'MissingParameterValue');
    end
end

% Warn when there are useless computation
if (do_estim_lnv) && (nargout < 2)
    warning (['stk_param_init will be computing an estimation of the ' ...
        'variance of the noise, perhaps should you call the function ' ...
        'with two output arguments?']);
end

%% Begin to propose values
covmo_matiso0 = covmo_matiso;
mat_iso = get(covmo_matiso, 'param');

model.param = covmo_matiso;
prior = get(covmo_matiso0, 'prior');

%% Check for special case: constant response
if (std (double (zi)) == 0)
    warning ('STK:stk_param_init:ConstantResponse', ...
        'Parameter estimation is impossible with constant-response data.');
    
    if ~strcmpi(get(covmo_matiso, 'func'), 'stk_materncov_iso')
        vect_param_default = log([1.0; 1.0]);
    else
        vect_param_default = log([1.0; 1.0; 5/2*d]);
    end
    
    % default values for param
    if isempty(prior)    % no prior distribution ==> return default values
        mat_iso = stk_set_optimizable_parameters (mat_iso, vect_param_default);
    elseif isa(prior, 'stk_prior_gaussian')
        % If gaussian prior : return the center of the distribution.
        [lb, ub] = stk_param_getdefaultbounds(prior);
        ind_default = logical(isinf(lb).*isinf(ub));
        % remove case where prior uncertainty is inf or parameter is inf
        
        vect_param = prior.mean;
        vect_param(ind_default) = vect_param_default(ind_default);
        mat_iso = stk_set_optimizable_parameters (mat_iso, vect_param);
    else
        [lb, ub] = stk_param_getdefaultbounds(prior);
        % TO DO : define a function 'mode' for stk_prior class
        vect_param = (lb + ub)/2;
        ind_default = logical(isinf(vect_param).*isnan(vect_param));
        
        vect_param(ind_default) = vect_param_default(ind_default);
        mat_iso = stk_set_optimizable_parameters (mat_iso, vect_param);
    end
    covmo_matiso0 = set(covmo_matiso0, 'param', mat_iso);
    
    if do_estim_lnv
        if isnumeric(noimo0) % in this case, lnv is a scalar
            noimo0 = 0;
        else
            model.param = covmo_matiso0;
            % FIXME: We should be calling stk_param_init_lnv here...
            noimo0 = stk_param_init_lnv_ (noimo0, model, xi, zi);
        end
    end
    return;  % Return some default values
end

%% List of possible values for the regularity parameter
if ~strcmpi(get(covmo_matiso, 'func'), 'stk_materncov_iso')
    % In this case, I am supposing that regularity is fixed, and the
    % parameter is an stk_param_iso
    lognu_list = +Inf;  % default value. Not used anymore
    lognu_opt = false;
else
    lognu_list = log(5/2*d);
    lognu_opt = true;
end

%% List of possible values for the range parameter
len_logacc_list = 5;

box_diameter = sqrt (sum (diff (box) .^ 2));

rho_max  = 2 * box_diameter;
rho_min  = box_diameter/50;
logacc_list = -linspace (log(rho_min), log(rho_max), len_logacc_list)';


%% List of possible values for the sigma2 parameter
if do_estim_lnv
    % General case when log-noise variance has to be estimated
    logsig2_list = log(var(zi)) + log (10.^((-3:3:3)'));
elseif stk_isnoisy(model)   % but do_estim_lnv is false
    if isnumeric (noimo0)
        nv = exp (noimo0);
    else
        nv = stk_covmat (model.lognoisevariance, xi, [], -1, true);
    end
    logsig2_list = log(mean(nv)) + log (10.^((0:3:6)'));
    
else % do not estim log-noise variance and the model is not noisy
    noimo0 = -Inf;      % assert not noisy model
    logsig2_list = 0.0; % we will provide the best value of sig2 according to rho
end

%% Put the list as grid
% Repmat parameters
[lognu_list, logacc_list, logsig2_list] = ndgrid(lognu_list, logacc_list, logsig2_list);
lognu_list   = lognu_list(:);
logacc_list  = logacc_list(:);
logsig2_list = logsig2_list(:);

%% Correct with prior distribution
if ~isempty(prior) && isa(prior, 'stk_prior')
    if lognu_opt
        matrix_param = [logsig2_list, logacc_list, lognu_list];
    else
        matrix_param = [logsig2_list, logacc_list];
    end
    
    matrix_param = stk_param_init_adapt_to_prior(prior, matrix_param);
    
    logsig2_list   = matrix_param(:, 1);
    logacc_list    = matrix_param(:, 2);
    if lognu_opt
        lognu_list = matrix_param(:, 3);
    end
end


%% Initialize parameter search
nu_best     = NaN;
rho_best    = NaN;
sigma2_best = NaN;
noimo_best  = noimo0;
aLP_best    = +Inf;

% Try all possible nu, rho and sig2 from the lists
for kll = 1:length(lognu_list)
    lognu = lognu_list(kll, 1);
    logacc = logacc_list(kll, 1);
    logsig2 = logsig2_list(kll, 1);
    
    if lognu_opt
        mat_iso.logreg  = lognu;
    end
    mat_iso.logacc  = logacc;
    mat_iso.logsig2 = logsig2;
    model.param = set(model.param, 'param', mat_iso);
    sigma2 = exp(logsig2);
    
    %% Calculate lonoisevariance if asked
    if (do_estim_lnv && isnumeric(noimo0)) || (~do_estim_lnv && ~stk_isnoisy(model))
        % Special algorithm : if log-noise var must be estimated, and is a
        % numeric value OR
        % if the model is not noisy
        
        mat_iso.sig2 = 1.0;   % fix sig2 to 1
        model.param = set(model.param, 'param', mat_iso);
        
        % Change log-noise variance
        if ~stk_isnoisy(model)
            model.lognoisevariance = -inf;
        else
            model.lognoisevariance = logsig2 - max(logsig2_list); % < 0
        end
        
        % Use the best value of sigma2
        [~, sigma2] = stk_param_gls (model, xi, zi);
        if ~(sigma2 > 0), continue; end
        
        % Scale both variances using the GLS estimate
        mat_iso.logsig2 = log(sigma2);
        model.param = set(model.param, 'param', mat_iso);
        model.lognoisevariance = model.lognoisevariance + log(sigma2);
        
    elseif do_estim_lnv && isa(noimo0.param, 'stk_param_cstlnv')
        % Special algorithm when the log-noise variance is made
        % with constant log-noise variance
        % FIXME: We should be calling stk_param_init_lnv here...
        model.lognoisevariance = stk_param_init_lnv_ (noimo0, model, xi, zi);
        % Use the best value of sigma2
        [~, sigma2] = stk_param_gls (model, xi, zi);
        if ~(sigma2 > 0), continue; end
        
        % Scale both variances using the GLS estimate
        mat_iso.logsig2 = log(sigma2);
        model.param = set(model.param, 'param', mat_iso);
        model.lognoisevariance.param.lognoivar =...
            model.lognoisevariance.param.lognoivar + log(sigma2);
        
    elseif do_estim_lnv %% && ~isnumeric(noimo0)
        % FIXME: We should be calling stk_param_init_lnv here...
        model.lognoisevariance = stk_param_init_lnv_ (noimo0, model, xi, zi);
    end % ~do_estim_lnv ==> nothing to change
    
    % Now, compute the antilog-posterior
    aLP = stk_param_mlogpost(model, xi, zi);
    if ~isnan(aLP) && (aLP < aLP_best)
        nu_best     = exp(lognu);
        rho_best    = exp(-logacc);
        aLP_best    = aLP;
        sigma2_best = sigma2;
        noimo_best  = model.lognoisevariance;
    end
end     % kll = 1:len_list


if isinf (aLP_best)
    errmsg = 'Couldn''t find reasonable parameter values... ?!?';
    stk_error (errmsg, 'AlgorithmFailure');
end


%% Return the best values
if lognu_opt
    mat_iso.reg = nu_best;
end
mat_iso.range = rho_best;
mat_iso.sig2  = sigma2_best;

covmo_matiso0 = set(covmo_matiso0, 'param', mat_iso);
noimo0 = noimo_best;
end