% CALL : diff = find_numb(param, name_field)
%
% CALL : diff = find_numb(param, name_field, numb)
%
% Return the indexes in the parameter vector corresponding to name_field.
% If umb is provided, it returns only the corresponding index, else, it
% returns all indexes.
%
% - param : an stk_parameter
% - name_fields : the field name of param which must be derivate
% - numb : an integer between 1 and length( get(param,
% name_fields_diff) ), indicating which number inside the field is looked for.
%
% * diff : an integer between 1 and length(param)
%
% Ex : dim = 4; param = stk_param_matern_aniso(0.1, 0.1*(1 + (1:dim)), 0.1*(dim + 2))
%   
% param = <stk_param_matern_aniso>
% 
% logsig2 : 0.1
%  logacc : 0.2
%           0.3
%           0.4
%           0.5
%  logreg : 0.6
%
%   find_numb(param, 'logsig2')              ==> 1
%   find_numb(param, 'logacc')               ==> [2; 3; 4; 5]
%   find_numb(param, 'logacc', [1; 4])       ==> [2; 5]
%   find_numb(param, 'logacc', [1; dim + 1]) ==> error !
%   find_numb(param, 'logperm')              ==> error !
%   find_numb(param, {'logsig2'})            ==> 1
%   find_numb(param, {'logsig2'; 'logacc'})  ==> error !
%
function diff = find_numb(param, name_field, numb)

if ~ischar(name_field)
    if iscellstr(name_field)
        if length(name_field) == 1
            name_field = name_field{1};
        else
           stk_error('Only one name at a time, do not use cell array.', 'IncorrectSize')  
        end
    else
        stk_error('''name_field'' must be a char.', 'InvalidArgument');
    end
end

%% Get back field names and sizes
[name_optim_fields, cumlength_optim_fields] = optimizable_fields(param);

index_fields = find(cellfun(@(string)(strcmpi(string, name_field)), name_optim_fields ));

if length(index_fields) ~= 1
    if length(index_fields) < 1
        stk_error(['There is not any optimizable field named ''', name_field,...
            ''' in a parameter of class ', class(param), '.'], 'InvalidArgument');
    else
        stk_error('Something wrong in your parameter (2 fields with the same name ?).',...
            'AlgorithmFailure');
    end
end
diff = (cumlength_optim_fields(index_fields) + 1):cumlength_optim_fields(index_fields + 1);
diff = diff';   % vector
if nargin > 2 && ~isempty(numb)
    diff = diff(numb);
end
end

