% CALL : [lb, ub] = stk_param_getdefaultbounds(par_bro, xi, zi)
%
% Get the lower bound and upper bound of optimization for a parameter.
% lb <= param <= ub.
% lb and ub are vector with the same length as param.
function [lb, ub] = stk_param_getdefaultbounds(par_bro, xi, zi)


if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Check size
if size(xi, 1) ~= size(zi, 1);
    stk_error('xi and zi does not have the same size.', 'IncorrectSize');
end

if size(zi, 2) > 1
    stk_error('zi must be a vector.', 'IncorrectSize');
end

if size(xi, 2) > 1
    warning('STK:stk_param_getdefaultbounds:xiNotVector',...
        'xi should be a vector. Use their norm-2 distances instead');
    xi = sqrt(diag(xi*xi'));
end

%% constants
opts = stk_options_get ('stk_param_getdefaultbounds');
TOLVAR = opts.tolvar;

%% Bounds for the variance parameter
log_empirical_variance = log (var (double (zi)));
logsig2 = par_bro.logsig2;
if isinf(log_empirical_variance) && log_empirical_variance < 0 % == - Inf
    logvar_lb = logsig2 - TOLVAR;
    logvar_ub = logsig2 + TOLVAR;
else
    logvar_lb = min (log_empirical_variance, logsig2) - TOLVAR;
    logvar_ub = max (log_empirical_variance, logsig2) + TOLVAR;
end

%% Bounds of the intercept
intercept = par_bro.intercept;
if isnan(intercept) || isinf(intercept)
    intercept = 0;
end
inter_lb = intercept;
inter_ub = intercept; % No optimization according to intercept
%% Bounds for the degree

ti = abs(xi - intercept);   % center xi, take their absolute value
zi = (zi - mean(zi)).^2;    % center zi, take their "variance"

beta = [ti, ones(size(ti, 1), 1)]\zi;   % linear regression

log_degr_exp = log(beta(1)); % rough estimation of the degree
logdegr = par_bro.logdegr;
if isinf(log_degr_exp) && log_degr_exp < 0 % == - Inf
    logdeg_lb = log_degr - 2.5;
    logdeg_ub = log_degr + 2.5;
else
    logdeg_lb = min(logdegr, log_degr_exp) - 2.5;
    logdeg_ub = max(logdegr, log_degr_exp) + 2.5;
end
% exp(2.5) = 12.5 => degree in [par_bro.logdegr/12.5; par_bro.logdegr*12.5]

%% Return bounds
lb = [logvar_lb; logdeg_lb; inter_lb];
ub = [logvar_ub; logdeg_ub; inter_ub];

end

