% CALL : kov = stk_materncov_aniso(mat_aniso, x, y)
%
% CALL : kov = stk_materncov_aniso(mat_aniso, x, y, diff)
%
% CALL : kov = stk_materncov_aniso(mat_aniso, x, y, diff, pairwise)
% [overload base function]
function kov = stk_materncov_aniso(mat_aniso, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 xs ys param0 pairwise0 D Kx_cache compute_Kx_cache

%% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

% check size consistency
dim = size (x, 2);
if (size (y, 2) ~= dim),
    stk_error ('xi and yi have incompatible sizes.', 'InvalidArgument');
end

%% extract parameters from the anisotropic Matern param
Sigma2 = get(mat_aniso, 'sig2');
Nu     = get(mat_aniso, 'reg');
invRho = get(mat_aniso, 'acc');

% check param
if (length (invRho) ~= dim)
    stk_error ('xi and mat_aniso have incompatible sizes.', 'InvalidArgument');
end

% check parameter values
if ~ (Sigma2 > 0) || ~ (Nu > 0) || ~ all (invRho >= 0),
    error ('Incorrect parameter value.');
end

%% check if all input arguments are the same as before
% (or if this is the first call to the function)
if isempty (x0) || isempty (y0) || isempty (param0) ...
        || ~ isequal ({x, y}, {x0, y0}) ...
        || ~ isequal([Sigma2; Nu; invRho], param0) ...
        || ~ isequal (pairwise, pairwise0)
    % compute the distance matrix
    xs = bsxfun (@times, x, invRho');
    ys = bsxfun (@times, y, invRho');
    D = stk_dist (xs, ys, pairwise);
    % save arguments for the next call
    x0 = x;  y0 = y;  param0 = [Sigma2; Nu; invRho];  pairwise0 = pairwise;
    % recomputation of Kx_cache is required
    compute_Kx_cache = true;
end

%% Compute the covariance matrix
if diff < 0
    %%% compute the value (not a derivative)
    kov = Sigma2 * stk_rbf_matern_local (Nu, D, -1);
else
    [diff_field, ind] = find_field(mat_aniso, diff);
    switch diff_field
        case 'logsig2'
            %%% diff wrt log(Sigma2)
            kov = Sigma2 * stk_rbf_matern_local (Nu, D, -1);
        case 'logreg'
            %%% diff wrt log(Nu)
            kov = Nu * Sigma2 * stk_rbf_matern_local (Nu, D, 1);
        case 'logacc'
            %%% diff wrt log(1/Rho(ind))
            if compute_Kx_cache || isempty (Kx_cache)
                Kx_cache = 1./(D+eps) .* (Sigma2 * stk_rbf_matern_local (Nu, D, 2));
                compute_Kx_cache = false;
            end
            if pairwise,
                kov = (xs(:, ind) - ys(:, ind)) .^ 2 .* Kx_cache;
            else
                kov = (bsxfun (@minus, xs(:, ind), ys(:, ind)')) .^ 2 .* Kx_cache;
            end
            
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
end

end % function

%% The same function as in STK, but stk_rbf_matern_local(2.5, h, 1) == 0
function k = stk_rbf_matern_local (nu, h, diff)

if nargin > 3,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

% default: compute the value (not a derivative)
if nargin < 3,
    diff = -1;
end

if ~ ((isscalar (nu)) && (nu > 0))
    stk_error ('nu should be a positive scalar.', 'IncorrectSize');
end

% Handle the case of VERY large nu
if nu > 1e305,
    % nu = 1e306 is so large thatn even gammaln (nu) is not defined !
    if diff <= 0,
        k = stk_rbf_gauss (h);  return;
    else
        % Cannot compute or approximate the derivative for such a large nu
        k = nan (size (h)); return;
    end
end

% Tolerance for the detection of half-integer values of nu
TOL = 10 * eps;

% We have no analytical expression for the derivative with respect to nu,
% even if nu is 3/2, 5/2 or +Inf
if diff ~= 1,
    
    if abs (nu - 1.5) < TOL,
        
        k = stk_rbf_matern32 (h, diff - 1);  return;
        
    elseif abs (nu - 2.5) < TOL,
        
        k = stk_rbf_matern52 (h, diff - 1);  return;
        
    end
    
end

[N, M] = size (h);
hp = abs (reshape (h, N * M, 1));
t = 2 * sqrt (nu) * hp;
z = 0.5 * exp (gammaln (nu) - nu * log (0.5 * t));
I = ~ isinf (z);

if diff <= 0
    
    k = zeros (N * M, 1);
    
    % When z < +Inf, compute using the modified Bessel function of the second kind
    k(I) = 1 ./ z(I) .* besselk_local (nu, t(I));
    
    % When z == +Inf, it means nu is large and/or t is close to zero.
    % We approximate the result with the upper bound provided by the Gaussian case.
    k(~I) = stk_rbf_gauss (h(~I));
    
elseif diff == 1  % numerical derivative wrt Nu
    
    if abs (nu - 2.5) < TOL, % set differential to 0 to accelerate estimation
        k = zeros (N * M, 1);
    else
        itermax = 2;
        delta = 1e-4;
        dk = zeros (N * M, itermax);
        for l= 1:itermax
            Nu_p = nu + 2 ^ (l - 1) * delta;
            Nu_m = nu - 2 ^ (l - 1) * delta;
            t_p = 2 * sqrt (Nu_p) * hp;
            t_m = 2 * sqrt (Nu_m) * hp;
            k_p = 1 / (2 ^ (Nu_p - 1) * gamma (Nu_p)) .* t_p(I) .^ Nu_p .* ...
                besselk_local (Nu_p, t_p(I));
            k_m = 1 / (2 ^ (Nu_m - 1) * gamma (Nu_m)) .* t_m(I) .^ Nu_m .* ...
                besselk_local (Nu_m, t_m(I));
            dk(I, l) =  k_p - k_m;
        end
        k = 1 / (12 * delta)* (- dk(:, 2) + 8 * dk(:, 1));
    end
    
elseif diff == 2  % deriv. wrt h
    
    k = zeros (N * M, 1);
    dtdh = 2 * sqrt (nu);
    k(I)  = - dtdh ./ z(I) .* besselk_local (nu - 1, t(I));
    
end

k = reshape (k, N, M);

end % function


function y = besselk_local (nu, x)

opts = stk_options_get('stk_rbf_matern');

if size(x, 1) < opts.min_size_for_parallelization,
    y = besselk(nu, x);
else
    y = stk_parallel_feval(@(t)(besselk(nu, t)), x, true, opts.min_block_size);
end

end % function
