% CALL : kov = stk_materncov_iso(mat_iso, x, y)
%
% CALL : kov = stk_materncov_iso(mat_iso, x, y, diff)
%
% CALL : kov = stk_materncov_iso(mat_iso, x, y, diff, pairwise)
% [overload base function]
function kov = stk_materncov_iso(mat_iso, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 param0 pairwise0 D

%% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

%% Extract parameters from the isotropic Matern parameter.
Sigma2 = get(mat_iso, 'sig2');
Nu     = get(mat_iso, 'reg');
invRho = get(mat_iso, 'acc');

% check parameter values
if ~ (Sigma2 > 0) || ~ (Nu > 0) || ~ (invRho >= 0),
    error ('Incorrect parameter value.');
end

%% check if all input arguments are the same as before
% (or if this is the first call to the function)
if isempty (x0) || isempty (y0) || isempty (param0) ...
        || ~ isequal ({x, y}, {x0, y0}) ...
        || ~ isequal([Sigma2; Nu; invRho], param0) ...
        || ~ isequal (pairwise, pairwise0)
    % compute the distance matrix
    D  = invRho * stk_dist (x, y, pairwise);
    % save arguments for the nex call
    x0 = x;  y0 = y;  param0 = [Sigma2; Nu; invRho];  pairwise0 = pairwise;
end

if diff < 0
    %%% compute the value (not a derivative)
    kov = Sigma2 * stk_rbf_matern (Nu, D, -1);
else
    diff_field = find_field(mat_iso, diff);
    switch diff_field
        case 'logsig2'
            %%% diff wrt log(Sigma2)
            kov = Sigma2 * stk_rbf_matern (Nu, D, -1);
        case 'logreg'
            %%% diff wrt log(Nu)
            kov = Nu * Sigma2 * stk_rbf_matern (Nu, D, 1);
        case 'logacc'
            %%% diff wrt - log(invRho)
            kov = D .* (Sigma2 * stk_rbf_matern (Nu, D, 2));
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
end

end % function

