% CALL : mf_cov = stk_param_multifidelity_cov(logsig2, logESR, logDecreOrder,...
%   logacc0, logaccE, logreg0, logregE, intercept)
%
% Build a multi-fidelity covariance parameter.
% Should be used for covariance of the form :
% k( (x;t), (x';t') ) = sig2*[ M{reg0}( ||(x - y).*acc0|| )
%                   + G*(min{t,t'}^{DecreOrder})*M{regE}( ||(x - y).*accE|| )].
% - logsig2 : logarithm of the variance at 0 of the covariance. Must be an
% unique numeric;
% - logESR : logarithm of the inverse of the Signal-to-Noise Ratio (where
% the "noise" is not a noise, but the error part) (Error-to-Signal ratio);
% - logdegr : logarithm of the decreasing degree. Indicate the order
% of magnitude of the decreasing of the error. Must be an unique numeric;
% - logacc0 : logarithm of the accuracies of the covariance of the ideal
% part. Accuracies are the inverse of ranges. Must be a vector;
% - logaccE : logarithm of the accuracies of the covariance of the error
% part. Must be a vector;
% - logreg0 : logarithm of the Matern covariance of the ideal part. Must be
% an unique numeric;
% - logregE : logarithm of the Matern covariance of the error part. Must be
% an unique numeric.
%
% Default value are NaN.
% Log-accuracy vectors must have the same size.
%
% CALL : mf_cov = stk_param_multifidelity_cov(param_matern0, param_maternE,
%   param_nonstat)
%
%   Compute the multi-fidelity covariance parameter with directly the good
%   parameters.
% - param_matern0 and param_maternE are 2 stk_param_matern_aniso objects,
% with same length;
% - param_nonstat is a stk_param_nonstatvar_cov, which will be used to
% describe the error evolution.
function mf_cov = stk_param_multifidelity_cov(varargin)

if nargin > 8
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

if nargin < 1 ...
        || isa(varargin{1}, 'stk_param')...
        || (nargin > 1 && isempty(varargin{1}) && isa(varargin{2}, 'stk_param'))...
        || (nargin > 2 && isempty(varargin{1})...
        && isempty(varargin{2}) && isa(varargin{3}, 'stk_param'))
    [mat_aniso0, mat_anisoE, param_nonstat] = parseInputs_param(varargin{:});
else
    [mat_aniso0, mat_anisoE, param_nonstat] = parseInputs_numeric(varargin{:});
end

%% Build the multi-fidelity covariance parameters
mf_cov = struct('param_Ideal', mat_aniso0,...
    'param_Error', mat_anisoE,...
    'param_Level', param_nonstat);
mf_cov = class(mf_cov, 'stk_param_multifidelity_cov', stk_param());
end

%% Parse inputs
function [mat_aniso0, mat_anisoE, param_nonstat] =...
    parseInputs_param(mat_aniso0, mat_anisoE, param_nonstat)

if nargin > 3
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Parameter of the level evolution
if nargin < 3 || isempty(param_nonstat)
    param_nonstat = stk_param_nonstatvar_cov(stk_param_matern_iso(log(1.0),...
        log(1.0), log(0.5)));
else
    if ~isa(param_nonstat, 'stk_param_nonstatvar_cov')
        stk_error(['The level evolution must be described by',...
            ' a non-stationnary covariance parameter.'], 'InvalidArgument');
    end
   
    if isfield(param_nonstat, 'logsig2')
        % fix the logarithm of the variance at 0 of the level parameter at
        % 1
        param_nonstat = set(param_nonstat, 'logsig2', log(1.0));
    end
end

%% Parameter of the error part
if nargin < 1 || (isempty(mat_aniso0) && (nargin < 2 || isempty(mat_anisoE)))
    mat_aniso0 = stk_param_matern_aniso();
    mat_anisoE = stk_param_matern_aniso();
    
elseif isempty(mat_aniso0)
    if ~isa(mat_anisoE, 'stk_param_matern_aniso')
        stk_error('The error part is described by an anisotropic Matern covariance.',...
            'InvalidArgument');
    end
    dim = length(mat_anisoE.logacc);
    mat_aniso0 = stk_param_matern_aniso(NaN, NaN(dim, 1));
    
elseif nargin < 2 || isempty(mat_anisoE)
    if ~isa(mat_aniso0, 'stk_param_matern_aniso')
        stk_error('The ideal part is described by an anisotropic Matern covariance.',...
            'InvalidArgument');
    end
    dim = length(mat_aniso0.logacc);
    mat_anisoE = stk_param_matern_aniso(NaN, NaN(dim, 1));
    
else
    if ~isa(mat_aniso0, 'stk_param_matern_aniso')
        stk_error('The ideal part is described by an anisotropic Matern covariance.',...
            'InvalidArgument');
    end
    if ~isa(mat_anisoE, 'stk_param_matern_aniso')
        stk_error('The error part is described by an anisotropic Matern covariance.',...
            'InvalidArgument');
    end
    
    if length(mat_aniso0.logacc) ~= length(mat_anisoE.logacc)
        stk_error('The dimension of both Matern covariance parameters are not equal',...
            'IncorrectSize');
    end
    
end

end


function [mat_aniso0, mat_anisoE, param_nonstat] =...
    parseInputs_numeric(logsig2, logESR, logdegr, logacc0, logaccE,...
    logreg0, logregE, intercept)

if nargin > 8
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Intercept
if nargin < 8 || isempty(intercept)
    intercept = 0;
end

%% Regularity
if nargin < 7 || isempty(logregE)
    logregE = NaN;    % Default value : no value
end

if nargin < 6 || isempty(logreg0)
    logreg0 = NaN;    % Default value : no value
end

%% Accuracies

if nargin < 4 || (isempty(logacc0) && (nargin < 5 || isempty(logaccE)))
    logacc0 = NaN;
    logaccE = NaN;  % default dimension : 1;
elseif isempty(logacc0)
    logaccE = logaccE(:);
    logacc0 = NaN(length(logaccE), 1);
elseif nargin < 5 || isempty(logaccE)
    logacc0 = logacc0(:);
    logaccE = NaN(length(logacc0), 1);
else
    logaccE = logaccE(:);
    logacc0 = logacc0(:);
    if length(logacc0) ~= length(logaccE)
        stk_error('Accuracy vectors must have the same length.', 'IncorrectSize');
    end
end

%% Other parameters
if nargin < 3 || isempty(logdegr)
    logdegr = NaN;    % Default value : no value
end

if nargin < 2 || isempty(logESR)
    logESR = NaN;    % Default value : no value
end

if nargin < 1 || isempty(logsig2)
    logsig2 = NaN;    % Default value : no value
end

%% Build parameters
mat_aniso0 = stk_param_matern_aniso(logsig2, logacc0, logreg0);
mat_anisoE = stk_param_matern_aniso(logESR,  logaccE, logregE);

param_nonstat = stk_param_nonstatvar_cov(...
    stk_param_matern_iso(log(1.0), log(1.0), log(0.5)), logdegr, intercept);

end
