% CALL : nvar = stk_multilevnoisecov_out(ml_lnv, x, diff)
%
% Estimate the noise variance in x, for the multi-level case.
% The last column of x must contain levels. Levels must be included in the
% multi-level log-noise parameter.
%
% - ml_lnv : a stk_param_multilevel_lnv parameter, which represent an
% multi-level noise variance parameter;
% - x : points where noise must be estimated. The last column of x must be
% in the levels of ml_lnv. Outside values cause an error;
% - diff : (default value : -1)  should indicate if we are
% asking for the value or its derivative. Can  be -1 or  between 1 and
% length(ml_lnv).
function nvar = stk_multilevnoisecov_out(ml_lnv, x, diff)

if nargin > 3,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

%% Check values
levels = ml_lnv.levels;

if any(isnan(levels))
    stk_error('Levels cannot have NaN values with this noise variance.', 'InvalidArgument');
end
x_level = x(:, end);
x_index = NaN(size(x, 1), 1);


for kl = 1:length(levels)
    index = (abs(x_level - levels(kl)) < 1e-8);
    x_index(index, 1) = kl;
end

if any(isnan(x_index))
    stk_error(['There are levels in ''x'' not included',...
        ' in the levels of the parameter.'], 'InvalidArgument');
end

%% Calculate noise variance
lnv = get(ml_lnv, 'lognoivar');
if (nargin < 3) || diff == -1   % default value for diff : -1
    nvar = exp(lnv(x_index));
    % lnv(x_index) : log-noise variance associated to the level
else
    [name_fields, nb_diff] = find_field(ml_lnv, diff);
    switch name_fields
        case 'lognoivar'
            index_diff = (x_index == nb_diff);
            
            nvar = zeros(size(x, 1), 1);
            nvar(index_diff, 1) = exp(lnv(nb_diff));
        case 'levels'
            % This covariance function cannot be optimized according to
            % levels.
            nvar = zeros(size(x, 1), 1);
        otherwise
            % No derivative ==> return 0
            nvar = zeros(size(x, 1), 1);
    end
end
                


end

