% CALL : [lb, ub] = stk_param_getdefaultbounds(ml_lnv, xi, zi)
% [overload stk_param function]
%
% Get the lower bound and upper bound of optimization for a parameter.
% lb <= param <= ub.
% lb and ub are vector with the same length as param.
function [lb, ub] = stk_param_getdefaultbounds(ml_lnv, xi, zi)

if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Check size
if size(zi, 2) ~= 1
    stk_error('zi is not a vector', 'IncorrectSize')
end
if size(zi, 1) ~= size(xi, 1)
    stk_error('zi and xi have not the same lenght.', 'IncorrectSize');
end
zi = double(zi);

%% Levels
xi_level = double(xi(:, size(xi, 2)));
levels = ml_lnv.levels;
nb_lev = length(levels);

%% Constants
TOLVAR = 0.5;
TOLEPS = 1e-5*range(levels);

lb_lev = min(levels)*ones(nb_lev, 1);
ub_lev = max(levels)*ones(nb_lev, 1);

%% Log-noise variances
lnv = get(ml_lnv, 'lognoivar');
lb_lnv = NaN(nb_lev, 1);
ub_lnv = NaN(nb_lev, 1);

for kl = 1:nb_lev
    indKl = abs(xi_level - levels(kl)) < TOLEPS;
    zi_kl = zi(indKl, :);
    
    if ~isempty(zi_kl)
        % Bounds for the variance parameter
        empirical_variance_kl = var(zi_kl);
        lb_lnv(kl, 1) = log (eps);
        ub_lnv(kl, 1) = log (empirical_variance_kl) + TOLVAR;
        if ~isnan(lnv(kl, 1))
            % Make sure that lnv falls within the bounds
            lb_lnv(kl, 1) = min(lb_lnv(kl, 1), lnv(kl, 1) - TOLVAR);
            ub_lnv(kl, 1) = max(ub_lnv(kl, 1), lnv(kl, 1) + TOLVAR);
        end
    else
        % If there are not observations at level kl
        lb_lnv(kl, 1) = log (eps);
        if ~isnan(lnv(kl, 1))
            ub_lnv(kl, 1) = lnv(kl, 1) + 0.5;
        end
    end
end

% Complete empty value
if any(isnan(ub_lnv))
    ub_lnv(isnan(ub_lnv), 1) = mean(ub_lnv(~isnan(ub_lnv)));
end

%% Transform the hyper-rectangle on hyper-square
lb_lnv = min(lb_lnv)*ones(nb_lev, 1);
ub_lnv = max(ub_lnv)*ones(nb_lev, 1);

%% Order the bounds
ml_lnv = set(ml_lnv, 'lognoivar', lb_lnv);
ml_lnv.levels = lb_lev;

lb = stk_get_optimizable_parameters(ml_lnv);    % return bounds in the good order

ml_lnv = set(ml_lnv, 'lognoivar', ub_lnv);
ml_lnv.levels = ub_lev;

ub = stk_get_optimizable_parameters(ml_lnv);    % return bounds in the good order

end

