% CALL : ml_lnv = stk_param_multilevel_lnv(lognoivar, levels)
%
% Build a multi-level log-noise variance parameters.
% This parameter should be used for noised multi-level models.
% - lognoivar : values of the log-noise variances;
% - levels : associated levels.
function ml_lnv = stk_param_multilevel_lnv(lognoivar, levels)

%% Check arguments
switch nargin
    case 0
        % If no inputs ==> 2 levels, and NaN variances
        cstlnv = stk_param_cstlnv(NaN(2, 1));
        levels = [1; 2];
        
    case 1
        if isempty(lognoivar)
            % correspond to case nargin == 0
            cstlnv = stk_param_cstlnv(NaN(2, 1));
        else
            cstlnv = stk_param_cstlnv(lognoivar);
        end
        % Default levels : [1; 2; ... ; nbLevels];
        levels = (1:length(cstlnv.lognoivar))';
        
    case 2
        if isempty(lognoivar)
            if isempty(levels)
                % Correspond to nargin == 0
                cstlnv = stk_param_cstlnv(NaN(2, 1));
                levels = [1; 2];
            else
                if ~isnumeric(levels)
                    try
                        levels = double(levels);
                    catch
                        stk_error('''levels'' must be double.', 'InvalidType');
                    end
                end
                
                levels = levels(:); % convert in vector
                levels = unique(levels, 'stable');  % assert unique value
                cstlnv = stk_param_cstlnv(NaN(length(levels), 1));
                % Default value for lognoivar : NaN at every level
            end
            
        else % if ~isempty(lognoisevar)
            cstlnv = stk_param_cstlnv(lognoivar);
            if isempty(levels)
                % correspond to the case nargin == 1
                levels = (1:length(cstlnv.lognoivar))';
            else
                if ~isnumeric(levels)
                    try
                        levels = double(levels);
                    catch
                        stk_error('''levels'' must be double.', 'InvalidType');
                    end
                end
                
                levels = levels(:); % convert in vector
                
                % Compare length of the 2 vectors
                if length(levels) > length(cstlnv.lognoivar)
                    % Complete the missing values
                    warning('STK:stk_param_multilevel_lnv:incorrectSize',...
                        'Levels and log-noise variance do not have the same length.');
                    cstlnv.lognoivar = [cstlnv.lognoivar;
                        NaN(length(levels) - length(cstlnv.lognoivar), 1)];
                elseif length(levels) < length(cstlnv.lognoivar)
                     stk_error(['Levels and log-noise variance',...
                         ' do not have the same length.'], 'IncorrectSize');
                end % else there are equal ==> all is OK.
                
            end % end if isempty(levels)
        end % end if isempty(lognoisevar)
        
    otherwise
        stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Build the multi-level log-noise variance parameter
ml_lnv = struct('levels', levels);
ml_lnv = class(ml_lnv, 'stk_param_multilevel_lnv', cstlnv);
end

