%STK_BROWNIAN_COV is the brownian covariance
% 
% CALL : kov = stk_brownian_cov(nonstat_cov, x, y)
%
% CALL : kov = stk_brownian_cov(nonstat_cov, x, y, diff)
%
% CALL : kov = stk_brownian_cov(nonstat_cov, x, y, diff, pairwise)
%
% Compute the brownian covariance with x and y, 1D vector, and nonstat_cov
% a stk_param_nonstatvar_cov.
%
% - nonstat_cov : a stk_param_nonstatvar_cov object. Its parameter must have
% a field named 'logsig2';
% - x, y : vectors where covariance must be estimated.
% - diff (default value : -1) : if diff == -1, return the standard
% covariance. Else, return the derivative of the covariance with respect to
% the diff-th parameter of par_bro.
% - pairwise (default value : false) : if true, return a vector of length
% the length of x and y : kov = (k(xi, yi)), 1 <= i <= length(x) == length(y).
% Else, return a matrix of size length(x) x length(y) :
%   kov = (k(xi, yj)), 1 <= i <= length(x); 1 = j <= length(y).
%
% The brownian covariance is not a real brownian motion, because it deforms
% the axis of time.
%   k(x, y) = sig2*min{|x - intercept|, |y - intercept|}^degree
function kov = stk_brownian_cov(nonstat_cov, x, y, diff, pairwise)

if nargin > 5,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

persistent x0 y0 degree0 pairwise0 M0L lnM0

%% process input arguments
x = double (x);
y = double (y);
if nargin < 4, diff = -1; end
if nargin < 5, pairwise = false; end

%% check consistency for inputs
dim = size (y, 2);
if dim ~= 1
    stk_error('The covariance accepts only 1D vectors.', 'IncorrectSize');
end

if (size (x, 2) ~= dim),
    stk_error ('x and y have incompatible sizes.', 'IncorrectSize');
end

%% Parameter
if isfield(nonstat_cov.param, 'logsig2')
    Sigma2 = exp(get(nonstat_cov.param, 'logsig2'));
else
    stk_error('The parameter of nonstat_cov must have a field ''logsig2''.',...
        'InvalidArgument'); 
end
degree = get(nonstat_cov, 'degr');
interc = get(nonstat_cov, 'intercept');

x = abs(x - interc);
y = abs(y - interc);    % transform the values

%% Compute useful data
% check if all input arguments are the same as before
% (or if this is the first call to the function)
if ( isempty(x0) || isempty(y0) || isempty (degree0) ...
        || ~isequal([x; y], [x0; y0])...
        || ~isequal(degree, degree0)...
        || ~isequal (pairwise, pairwise0) )
    
    if ~pairwise
        nx = length(x);
        ny = length(y);
        % M = min(repmat(x, 1, ny), repmat(y', nx, 1));
        
        lx = log(x);
        ly = log(y);
        lnM0 = min(repmat(lx, 1, ny), repmat(ly', nx, 1));
        
        xd = x.^degree;
        yd = y.^degree;
        M0L = min(repmat(xd, 1, ny), repmat(yd', nx, 1));
    else%pairwise == true
        M = min(x, y);
        M0L = M.^degree;
        lnM0 = log(M);
    end
    
    lnM0(isinf(lnM0)) = 0;
    
    x0 = x;  y0 = y;  degree0 = degree;  pairwise0 = pairwise;
end

%% Compute covariance
if diff == - 1
    kov = Sigma2*M0L;
else
    diff_field = find_field(nonstat_cov, diff);
    switch diff_field
        case 'logsig2'
            % diff wrt log(Sigma2)
            kov = Sigma2*M0L;
        case 'logdegr'
            % diff wrt to log(degree)
            kov = (Sigma2*degree)*lnM0.*M0L;
        case 'intercept'
            % No derivative wrt to intercept ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
        otherwise
            % No derivative ==> return 0
            if pairwise
                kov = zeros(size(x, 1), 1);
            else
                kov = zeros(size(x, 1), size(y, 1));
            end
    end
end
end

