% nonstat_cov = stk_param_nonstatcov_var(param, logdegr, intercept)
% 
% Create a non-stationnary covariance parameter, where the variance at 0
% varies.
%
% Should be used for covariance such as :
% k(x,y) = [|(x(:,end) - intercept)*(y(:,end) - intercept)|^degree] * r(x,y; param)
%
% - param : a stk_parameter;
% - logdegr : logarithm of the degree of evolution of the variance at 0;
% - intercept : (default value : 0) : the value of the intercept of the
% variance (value where variance is 0).
function nonstat_cov = stk_param_nonstatvar_cov(param, logdegr, intercept)

if nargin > 3
    stk_error('Too many input argument.', 'TooManyInputArgs');
end

%% Intercept
if nargin < 2 || isempty(intercept)
    intercept = 0;    % Default value : 0
else
    if ~isnumeric(intercept)
        try
            intercept = double(intercept);
        catch
            stk_error('''intercept'' must be a double.', 'InvalidType');
        end
    end

    if numel(intercept) > 1
        stk_error('There should be only one value for ''intercept''.', 'IncorrectSize');
    end
end
% NaN and Inf value are tolerated, but they should be fixed before usin
% gthe default covariance.
    
%% Degree
if nargin < 2 || isempty(logdegr)
    logdegr = NaN;    % Default value : no value
else
    if ~isnumeric(logdegr)
        try
            logdegr = double(logdegr);
        catch
            stk_error('''logdegr'' must be a double.', 'InvalidType');
        end
    end

    if numel(logdegr) > 1
        stk_error('There should be only one value for ''logdegr''.', 'IncorrectSize');
    end
end

%% Variance at 0
if nargin < 1 || isempty(param)
    param = stk_param_matern_iso(); % NaN values
else
    if ~isa(param, 'stk_param')
        stk_error('param must be a stk_param object.', 'InvalidArgument');
    end
end

%% Build the parameter
nonstat_cov = struct('param', param, 'logdegr', logdegr, 'intercept', intercept);
nonstat_cov = class(nonstat_cov, 'stk_param_nonstatvar_cov', stk_param());

end

