% CALL : logpri = set(logpri, propname, value)
% [overload stk_prior function]
function gauss_lp = set(gauss_lp, propname, value)

switch propname
    case 'mean'
        % Check class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('The mean vector must be a double.', 'InvalidArgument');
            end
        end
        % Check size
        if ~isvector(value)
            stk_error('The mean vector must be a vector.', 'InvalidArgument');
        end
        value = value(:); % assert column vector
        if length(value) ~= dimension(gauss_lp)
            stk_error(['The proposed new value of mean does not have',...
                ' the same size as the old value.'], 'IncorrectSize');
        end
        % No NaN values
        if any(isnan(value))
            stk_error('The mean vector must not have NaN values.', 'matrixWithNaN');
        end
        % No Inf values
        if any(isinf(value))
            stk_error('The mean vector must not have Inf values.', 'matrixWithInf');
        end
        
        gauss_lp.mean = value;
    case 'eigenvals'
        % Check class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('The eigen values must be a double.', 'InvalidArgument');
            end
        end
        % Check size
        if ~isvector(value)
            stk_error('The eigen values must form a vector.', 'InvalidArgument');
        end
        value = value(:); % assert column vector
        if length(value) ~= dimension(gauss_lp)
            stk_error(['The proposed new values of eigen values does not have',...
                ' as numerous as the old values.'], 'IncorrectSize');
        end
        % No NaN values
        if any(isnan(value))
            stk_error('The eigen values must not have NaN values.', 'matrixWithNaN');
        end
        % Positivity
        if any(value < 0)
            stk_error('The covariance matrix must be positive.', 'InvalidArgument');
        end
        
        gauss_lp.eigenvals = value;
    case 'eigenvect'
        % Check class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('The eigen vectors must be a double.', 'InvalidArgument');
            end
        end
        % Check size
        if size(value, 1) ~= size(value, 2)
            stk_error('The eigen vectors must be a square matrix.', 'InvalidArgument');
        end
        dim = dimension(gauss_lp);
        if size(value, 1) ~= dim
            stk_error(['The proposed new values of eigen vectors does not have',...
                ' the same dimension as the old values.'], 'IncorrectSize');
        end
        % No NaN values
        if any(isnan(value(:)))
            stk_error('The eigen values must not have NaN values.', 'matrixWithNaN');
        end
        % No Inf values
        if any(isinf(value(:)))
            stk_error('The eigen values must not have Inf values.', 'matrixWithInf');
        end
        % Orthogonality
        if ~stk_isequal_tolabs(value*value', eye(dim))
            stk_error('The eigen vectors must be orthogonal.', 'InvalidArgument');
        end
        
        gauss_lp.eigenvect = value;
        %% Other values returned by get function
    case 'covariance'
        if size(value, 1) == size(value, 2) % Square case
            [eigenvect, eigenvals] = eig(value);
        elseif isvector(value)              % case vector
            eigenvals = diag(value);
            eigenvect = eye(length(value));
        else
            stk_error(['Covariance must be a square matrix or',...
                ' a vector for diagonal matrix.'], 'IncorrectSize');
        end
            
        gauss_lp = set(gauss_lp, 'eigenvals', diag(eigenvals));
        gauss_lp = set(gauss_lp, 'eigenvect', eigenvect);
    case 'invcov'
        if size(value, 1) == size(value, 2) % Square case
            [eigenvect, invgenvals] = eig(value);
        elseif isvector(value)              % case vector
            invgenvals = diag(value);
            eigenvect = eye(length(value));
        else
            stk_error(['Inverse covariance must be a square matrix or',...
                ' a vector for diagonal matrix.'], 'IncorrectSize');
        end
        gauss_lp = set(gauss_lp, 'eigenvals', 1./diag(invgenvals));
        gauss_lp = set(gauss_lp, 'eigenvect', eigenvect);
        
    case 'variance'
        gauss_lp = set(gauss_lp, 'covariance', diag(value(:)));
        
    case 'invvar'
        gauss_lp = set(gauss_lp, 'invcov', diag(value(:)));
        
    case 'std'
        gauss_lp = set(gauss_lp, 'variance', value.^2);
        
    otherwise
        stk_error(['A gaussian log-prior does not have any property',...
            ' whose name is ', propname, '.'], 'UnknownField');
end

