% CALL : gauss_lp = stk_equalize_param(gauss_lp, ind)
%
% CALL : gauss_lp = stk_equalize_param(gauss_lp, ind, var_ind)
%
% CALL : gauss_lp = stk_equalize_param(gauss_lp, ind, var_ind, cov_ind)
%
% CALL : gauss_lp = stk_equalize_param(gauss_lp, ind, var_ind, cov_ind, mean_ind)
%
% Impose that every parameter(ind) are equals : param(ind) =
% param(1)*ones(length(ind), 1)
%
% - gauss_lp : a Gaussian log-prior;
% - ind : the index of parameters which must be equalized. Must be included
% between 1 and the dimension of the Gaussian log-prior;
% - var_ind : a unique number. The prior variance of the parameters. If not
% provided, the default value is +Inf;
% - cov_ind : the correlation between the equalized parameters and others.
% Must be size (dimension(gauss_lp) - length(ind)) x 1. The default
% value is 0;
% - mean_ind : a unique number. The prior value of the equalized
% parameters. Default value : the mean of previous mean prior parameters.
function gauss_lp = stk_equalize_param(gauss_lp, ind, var_ind, cov_ind, mean_ind)

if nargin > 5
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Check index
ind = unique(ind(:));
dim = dimension(gauss_lp);
if any(ind < 1) || any(ind > dim)
    stk_error(['Index must be between 1 and the dimension of',...
        ' the Gaussian log-prior.'], 'InvalidArgument');
end
len_ind = length(ind);
if len_ind == 1
    warning('STK:stk_equalize_param:impossibleToEqualized',...
        'There is only one parameter to equalize.');
end

%% Check other values
% variance
if nargin < 3 || isempty(var_ind)
    var_ind = +Inf;
else
    if numel(var_ind) > 1
        stk_error('Only one value for the variance.', 'IncorrectSize');
    end
    if isnan(var_ind)
        stk_error('NaN value are not allowed.', 'matrixWithNaN');
    end
    if var_ind < 0
        stk_error('A variance must be positive.', 'InvalidArgument');
    end
end
% covariance
if nargin < 4 || isempty(cov_ind)
    cov_ind = zeros(dim - len_ind, 1);
else
    if (~isvector(cov_ind)) || (length(cov_ind) ~= (dim - len_ind))
        if numel(cov_ind) == 1
            cov_ind = cov_ind*ones(dim - len_ind, 1);
        else
            stk_error(['The correlation must be a vector of length dim - length(ind) = ', ...
                num2str(dim - length(ind)), '.'], 'IncorrectSize');
        end
    end
    cov_ind = cov_ind(:);
    if isnan(cov_ind)
        stk_error('NaN value are not allowed.', 'matrixWithNaN');
    end
end
% mean
if nargin < 5 || isempty(mean_ind)
    mean_ind = mean(gauss_lp.mean(ind));
else
    if numel(mean_ind) > 1
        stk_error('Only one value for the mean.', 'IncorrectSize');
    end
    if isnan(mean_ind)
        stk_error('NaN value are not allowed.', 'matrixWithNaN');
    end
end

%% Change values
gauss_lp.mean(ind) = mean_ind;  % Change mean

% Index not equalizes
not_ind = (1:dim)';
not_ind = not_ind(~ismember(not_ind, ind));
S = get(gauss_lp, 'covariance');

if isequal(cov_ind, zeros(dim - len_ind, 1)) % separate subspace
    % Equalized index subspace
    [eigvect_ind, eigvals_ind] = eig(ones(len_ind));
    eigvals_ind = var_ind*diag(eigvals_ind);
    eigvals_ind(isnan(eigvals_ind)) = 0;    % Case 0 x + Inf : case var_ind = +Inf
    
    % Complementary subspace
    S_notInd = S(not_ind, not_ind);
    [eigvect_notInd, eigvals_notInd] = eig(S_notInd);
    
    % New values
    eigenvect = zeros(dim, dim);
    eigenvals = NaN(dim, 1);
    
    eigenvect(ind, ind) = eigvect_ind;
    eigenvect(not_ind, not_ind) = eigvect_notInd;
    % Other values are 0;
    eigenvals(ind, 1) = eigvals_ind;
    eigenvals(not_ind, 1) = diag(eigvals_notInd);
    
    % Change calues
    gauss_lp.eigenvals = eigenvals;
    gauss_lp.eigenvect = eigenvect;
    
elseif ~isinf(var_ind)  % easy case
    S(ind, ind) = var_ind*ones(len_ind);            % fix variance
    S(not_ind, ind) = repmat(cov_ind , 1, len_ind); % fix covariance
    S(ind, not_ind) = repmat(cov_ind', len_ind, 1);
    
    gauss_lp = set(gauss_lp, 'covariance', S);
else
    stk_error(['The algorithm cannot treat the case with an infinite',...
        ' variance and correlation with other parameters.'], 'AlgorithmFailure');
end
end

