% stk_param_mlogpost computes the restricted likelihood of a model given data
%
% CALL: [ALP, dALP_dtheta, dALP_dLNV] = stk_param_mlogpost(MODEL, XI, YI)
%
%   computes the opposite of the log-posterior likelihood (denoted by ALP for
%   Anti-Log Posterior) of MODEL given the data (XI, YI). The function
%   also returns the gradient dALP_dtheta of ALP with respect to the parameters
%   of the covariance function and the derivative dALP_dLNV of ALP with respect
%   to the logarithm of the noise variance.
%
% The log-posterior is calculated as the anti-restricted likelihood
% (stk_param_relik), plus the minus log-prior of parameters.
% This second part is calculated only if model.param is a stk_covmodel,
% and/or if model.lognoisevariance is a stk_noisemodel.
% In both case, it will be calculated with the stk_mlogprior function.
% If model.param.prior or model.lognoisevariance.prior is empty, the
% associated prior will be supposed None.
%
% Warning : because of supporting oldest version, taking account prior can
% be redundant if model has fields 'prior', or 'noiseprior'.
function [lpost, dlpost_cov_param, dlpost_noise_param] = stk_param_mlogpost(model, xi, yi, vect_param)

if nargin > 4
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

%% If a vector of parameter is provided, change it in the model
if nargin > 3 && ~isempty(vect_param)
    model = stk_set_optimizable_parameters(model, vect_param);
end

%% First calculate the Anti-restricted Likelihood
switch nargout
    case {0, 1}
        lpost = stk_param_relik (model, xi, yi);
    case 2
        [lpost, dlpost_cov_param] = stk_param_relik (model, xi, yi);
        
    case 3
        [lpost, dlpost_cov_param, dlpost_noise_param] = stk_param_relik (model, xi, yi);
        
    otherwise
        stk_error('Too many output arguments.', 'TooManyOutputArgs');
end

%% Compute prior of covariance parameter
if isa(model.param, 'stk_covmodel') && ~isempty(model.param.prior)
    switch nargout
        case {0, 1}
            lprior_covparam =...
                stk_mlogprior(model.param.prior, model.param.param);
            
            lpost = lpost + lprior_covparam;
            
            
        case {2, 3} % no derivative along log noise variance parameters.
            [lprior_covparam, dlprior_cov_param] =...
                stk_mlogprior(model.param.prior, model.param.param);
            
            lpost = lpost + lprior_covparam;
            dlpost_cov_param = dlpost_cov_param + dlprior_cov_param;
            
            
        otherwise
            stk_error('Too many output arguments.', 'TooManyOutputArgs');
    end
end

%% Calculated prior of noise-variance parameter
if isa(model.lognoisevariance, 'stk_noisemodel') && ~isempty(model.lognoisevariance.prior)
    switch nargout
        case {0, 1, 2}% no derivative along log noise variance parameters.
            lprior_lnv_param =...
                stk_mlogprior(model.lognoisevariance.prior, model.lognoisevariance.param);
            
            lpost = lpost + lprior_lnv_param;
            
            
        case 3
            [lprior_param, dlprior_lnv_param] =...
                stk_mlogprior(model.lognoisevariance.prior, model.lognoisevariance.param);
            
            lpost = lpost + lprior_param;
            dlpost_noise_param = dlpost_noise_param + dlprior_lnv_param;
            
            
        otherwise
            stk_error('Too many output arguments.', 'TooManyOutputArgs');
    end
end

end

