% STK_SUR_PEXCEEDTHRES computes a Stepwise Uncertainty Reduction criterion.
%
% CALL: CRIT_VAL = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT)
%
%   computes the criteria SUR as the reduction uncertainty of the mean
%   pointwise probability of misclassification. More exactly, the criterion
%   computes :
%       E[mean{Var[Phi((Z{XU} - THRES)/noisevariance(XU))| (XI, ZI))}|
%       XI(ni + 1, :) = XT]
%   This criterion and the computation are provided in Chevalier (2014). It
%   is used to find observation points which can improve the knowledge of
%   the function around the threshold.
%
%   - THRES : the target threshold;
%   - XU : some points of the input space. Should be simulated according to
%   their input distributions. If there is not any distribution, sample
%   them according to a space-filling thecnics.
%       size(XU) = [nu, dim].
%   The greater nu is, the best the criteria will be, but the slower the
%   function is;
%   - MODEL : the model of your (unknown) function.
%       MODEL.dim = dim;
%   The dimension of MODEL must be the same as the dimension of XU.
%   - (XI, ZI) : some observations (possibly noised) of your function.
%       size(XI) = [ni, dim];
%       size(ZI) = [ni, 1].
%   The dimension of XI must be the same as the dimension of XU.
%   ZI must be a vector with the same length as the number of points XI;
%
%   - XT : some candidate points where the criteria must be computed.
%       size(XT) = [nt, dim].
%   The dimension of XT must be the same as the dimension of XU.
%
%   * CRIT_VAL : the value of the criteria at each candidate points.
%       size(CRIT_VAL) = [nt, 1]
%
% CALL: CRIT_VAL = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XYT)
%
%   computes the SUR criterion, but on the assumption that q points are
%   added to the MODEL (and not one). In this case, XYT is size [nt, q*dim],
%   where nt is the number of possibilities. Thus, at a line it, XYT(it, :)
%   = [X_{n+1}(it)', X_{n+2}(it)', ... X_{n+q}(it)']. The dimension of XYT
%   must be a multiple of the dimension of the problem.
%
% CALL: CRIT_VAL = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI)
%
%   computes the SUR criterion, but on the assumption that no point is
%   added to the model. The returned value is the current measure of
%   uncertainty
%
% CALL: CRIT_VAL = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT, WEIGHTS)
%
%   does the same thing, but instead of using a mean at the end of the
%   computation, return a weighted sum of the local criterion:
%   E[sum{ WEIGHTS.*Var[Phi((Z{XU} - THRES)/noisevariance(XU))| (XI, ZI)) }|
%       XI(ni + 1, :) = XT].
%   WEIGHTS must be a vector of length nu, the first dimension of XU.
%   The default values of weight is ones(nu, 1)/nu.
%
% CALL: CRIT_VAL = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT, ...
%       WEIGHTS, SIMPARAM)
%
%   does the same thing, but compute the mean on SIMPARAM. SIMPARAM
%   is a nb x dim_param matrix, such as each row is an hyper-parameter
%   of the model sampled according to the posterior distribution.
%
% CALL: [CRIT_VAL, RED_VAL] = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT)
% CALL: [CRIT_VAL, RED_VAL] = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT, WEIGHTS)
% CALL: [CRIT_VAL, RED_VAL] = stk_sur_pexceedthres(THRES, XU, MODEL, XI, ZI, XT,...
%       WEIGHTS, SIMPARAM)
%
%   computes also the uncertainty reduction. The sum of CRIT_VAL and
%   RED_VAL is a constant equals to the current measure of uncertainty.
%   If XT is not provided, or is empty, the reduction is zero.
%
% REFERENCES
%
%   [1] Clement Chevalier, Julien Bect, David Ginsbourger, Emmanuel
%       Vazquez, Victor Picheny, and Yann Richet.
%       Fast parallel kriging based stepwise uncertainty reduction with
%       application to the identication of an excursion set.
%       Technometrics, 56(4):455-465, 2014.
%
%   [2] Julien Bect, David Ginsbourger, Ling Li, Victor Picheny, and
%       Emmanuel Vazquez.
%       Sequential design of computer experiments for the estimation of a
%       probability of failure.
%       Statistics and Computing, 22(3):773�793, 2012.
%
% See also : stk_pmisclass, stk_sur_pruning
function [crit_val, red_val] = stk_sur_pexceedthres(thres, xu, model,...
    xi, zi, xt, weights, simParam)

if nargin > 8
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

compute_reduction = (nargout > 1);

% Default value for weights
if nargin < 7
    weights = [];
end

% Default value for xt
if nargin < 6
    xt = [];
end

%% Compute the criteria
if nargin < 8 || isempty(simParam)
    % Compute the criteria under the Gaussian process assumption
    if compute_reduction
        [crit_val, red_val] = stk_sur_pexceedthres_GP(thres, xu, model, xi, zi, xt, weights);
    else
        crit_val = stk_sur_pexceedthres_GP(thres, xu, model, xi, zi, xt, weights);
    end
else % nargin == 8 && ~isempty(simParam)
    % Compute the mean criterion over the parameters
    
    nt = size(xt, 1);
    crit_val = zeros(max(nt, 1), 1);
    
    % Hyper-parameter uncertainty
    % nu = size(xu, 1);
    % mean_prob = zeros(nu, 1);
    % var_prob = zeros(nu, 1);
    
    if compute_reduction
        % Data for reduction
        red_val = zeros(max(nt, 1), 1);
    end
    
    %% Loop on every parameters
    for kpar = 1:size(simParam, 1)
        % Change parameters of the model
        model = stk_set_optimizable_parameters(model, simParam(kpar, :));
        
        % Compute the criteria : use the assumption of Gaussian process
        %   under known parameters
        if compute_reduction
            [crit_val_kpar, red_val_kpar] = stk_sur_pexceedthres_GP(thres,...
                xu, model, xi, zi, xt, weights);
            % mean reduction (reccursive formulation)
            red_val = stk_mean_recursive(kpar, red_val, red_val_kpar);
            % E_n[ Reduction(xt, Theta) |Theta)]
        else
            crit_val_kpar = stk_sur_pexceedthres_GP(thres,...
                xu, model, xi, zi, xt, weights);
        end
        % mean crit (reccursive formulation)
        crit_val = stk_mean_recursive(kpar, crit_val, crit_val_kpar);
        % E_n[ Future uncertainty(xt, Theta) |Theta)]
        
        % % Compute the probability of exceeding the threshold at xu
        % zp_u = stk_predict(model, xi, zi, xu);
        % if stk_isnoisy(model)
        %     zp_u.var = zp_u.var + stk_covmat_noise (model, xu, [], -1, true);
        % end
        % prob_u = stk_distrib_normal_cdf(zp_u.mean, thres, sqrt(zp_u.var));
        % % Prob_u = E_n(P|Theta)
        
        % % Update the first and second moments
        % [mean_prob, var_prob] = stk_update_recursive(kpar, mean_prob, var_prob, prob_u);
        % % Var_prob = Var_n[E_n(P|Theta)]
        
    end % end for each parameters
    
    % %% Add the variance with respect to hyper-parameters
    % if isempty(weights)
    %     crit_val = crit_val + mean(var_prob);  % mean on xu/var on hyper-parameters
    % else
    %     crit_val = crit_val + sum(weights.*var_prob);
    % end
end

end

function [crit_val, red_val] = stk_sur_pexceedthres_GP(thres, xu, model, xi, zi, xt, weights)
% GP: Gaussian process

if nargin > 7
    stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Check inputs arguments

thres = double(thres);

xu = double(xu);
xi = double(xi);
zi = double(zi);
xt = double(xt);

nu  = size(xu, 1);
dim = size(xu, 2);
if size(xi, 2) ~= dim
    stk_error('xi and xu do not have the same dimension.', 'IncorrectSize');
end

if model.dim ~= dim
    stk_error('The inputs and the model do not have the same dimension.',...
        'IncorrectSize');
end

ni = size(xi, 1);
if ~isequal(size(zi), [ni, 1])
    stk_error(['zi must be a column vector with the same length as',...
        ' size(xi, 1).'], 'IncorrectSize');
end

if isempty(xt)
    % No observations are added
    nt = 1;
else
    nt = size(xt, 1);
end

if ~isempty(weights)
    weights = double(weights);
    weights = weights(:);	% assert columns vector
    
    if length(weights) == 1
        % Stupid case where there is only one weight. In this case, assume
        % the same weight for all xu
        weights = weights*ones(nu, 1);
    elseif length(weights) ~= nu % && length(weights) ~= 1
        stk_error(['The weight vector must have the same length, '...
            'as the integration points.'], 'IncorrectSize');
    end
    % Assume that the product zu.*weights make sense
    % size(weights) = [nu, 1];
    % size(WEIGHTS) = [nu, nt];
    WEIGHTS = repmat(weights, 1, nt);
end

%% Compute the variance of the future mean
% m_{n+1}(xu) | {x_{n+1} = xt(:, 1:dim), x_{n+2} = xt(:, dim +
% (1:dim)), ..., x_{n+q} = xt(:, dim*(q - 1) + (1:dim))}
%   ~ N(m_{n}(xu), nu_{n+1}(x_u))
nu_nPq_u = stk_variance_reduction(xu, model, xi, xt);

%% Compute useful data

% zp_u.var | x_{n+1} = xt
zp_u = stk_predict(model, xi, zi, xu);

% zp_u.mean - thres
diff_to_thres_u = zp_u.mean - thres;

%% Compute criterion
if stk_isnoisy(model)
    % Variance of observations (with noise)
    var_n_u = zp_u.var + stk_covmat_noise (model, xu, [], -1, true);
else
    var_n_u = zp_u.var;
end

diff_to_thres_u = diff_to_thres_u./sqrt(var_n_u);

% Compute the term of the bivnorm cumulative distribution function
correl2 = nu_nPq_u./repmat(var_n_u, 1, nt);         % size : [nu, nt]
% nu/v

if nu > 1 % Standard case
    diff_to_thres_u = repmat(diff_to_thres_u, 1, nt);	% size : [nu, nt]
    % Different behavior of the algorithm for correl2 == 0
    % Phi_2([x, y], m, mu, s, sig, 0) = Phi(x, m, s)*Phi(y, mu, sig)
    
    if stk_isnoisy(model)
        points_to_bivnorm = cat(3, diff_to_thres_u, diff_to_thres_u);   % size : [nu, nt, 2]
        
        % k/v
        correl1 = repmat(zp_u.var./var_n_u, 1, nt);	% size : [nu, nt]
        % Pointwise Uncertainty Reduction
        futur_unc = stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, correl1);
        futur_unc = futur_unc - stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, correl2);
        % Compute Phi2(dz/sqrt(v), dz/sqrt(v), k/v) - Phi2(dz/sqrt(v), dz/sqrt(v), nu/v)
        % v = k + l
        
    else % if no noise => correl1 = 1 => Simplification
        % Compute the term of the bivnorm cumulative distribution function
        points_to_bivnorm = cat(3, diff_to_thres_u, -diff_to_thres_u); % size : [nu, nt, 2]
        
        % Pointwise Uncertainty Reduction
        futur_unc = stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, -correl2);
        % Compute Phi2(dz/sqrt(v), -dz/sqrt(v), -nu/v)
        %   = Phi2(dz/sqrt(v), dz/sqrt(v), v/v) - Phi2(dz/sqrt(v), dz/sqrt(v), nu/v)
        % k = v <=> var_n_u = zp_u.var
    end
    % Size(futur_unc) = [nu, nt]
    
    % Criterion : Global Uncertainty Reduction
    if isempty(weights)
        % Assume weights = 1/nu
        crit_val = mean(futur_unc)';
    else    % Transpose, to return a column vector.
        crit_val = sum(WEIGHTS.*futur_unc)';
    end
    
    %% Reduction of uncertainty
    if nargout > 1
        points_to_rebivnorm = cat(3, diff_to_thres_u, diff_to_thres_u);  % size : [nu, nt, 2]
        futur_red = stk_distrib_bivnorm_cdf(points_to_rebivnorm, 0, 0, 1, 1, correl2)...
            - stk_distrib_normal_cdf(diff_to_thres_u, 0, 1).^2;
        % Remark: if correl2 = 0; futur_red = 0. However, there can be some
        % numerical instabilities which leads to a non-zero difference.
        
        if isempty(weights)
            red_val = mean(futur_red)';
        else
            red_val = sum(WEIGHTS.*futur_red)';
        end
    end
    
else % nu == 1, only one point in the integration point ??? ...
    % Particular case, because "stk_distrib_bivnorm_cdf" and "mean" have a
    % different behavior in this case
    
    correl2 = shiftdim(correl2, 1);   % reshape correl2 in a [nt, 1] vector
    
    % Different behavior of the algorithm for correl2 == 0
    % Phi_2([x, y], m, mu, s, sig, 0) = Phi(x, m, s)*Phi(y, mu, sig)
    if stk_isnoisy(model)
        points_to_bivnorm = repmat([diff_to_thres_u, diff_to_thres_u], nt, 1);
        % size : [nt, 2]
        
        % k/v
        correl1 = zp_u.var./var_n_u;	% size : [1, 1]
        % Pointwise Uncertainty Reduction
        crit_val = stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, correl1);
        crit_val = crit_val - stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, correl2);
        % Compute Phi2(dz/sqrt(v), dz/sqrt(v), k/v) - Phi2(dz/sqrt(v), dz/sqrt(v), nu/v)
        % v = k + l
        
    else % if no noise => correl1 = 1 => Simplification
        % Compute the term of the bivnorm cumulative distribution function
        points_to_bivnorm = repmat([diff_to_thres_u, -diff_to_thres_u], nt, 1);
        % size : [nt, 2]
        
        % Pointwise Uncertainty Reduction
        crit_val = stk_distrib_bivnorm_cdf(points_to_bivnorm, 0, 0, 1, 1, -correl2);
        % Compute Phi2(dz/sqrt(v), -dz/sqrt(v), -nu/v)
        %   = Phi2(dz/sqrt(v), dz/sqrt(v), v/v) - Phi2(dz/sqrt(v), dz/sqrt(v), nu/v)
        % k = v <=> var_n_u = zp_u.var
    end
    % Size(futur_unc) = [nu, nt]
    
    % Criterion : Uncertainty Reduction
    if ~isempty(weights)
        % Transpose, to return a column vector.
        crit_val = WEIGHTS'.*crit_val;
    end
    
    %% Reduction of uncertainty
    if nargout > 1
        points_to_rebivnorm = repmat([diff_to_thres_u, diff_to_thres_u], nt, 1);
        % size : [nt, 2]
        red_val = stk_distrib_bivnorm_cdf(points_to_rebivnorm, 0, 0, 1, 1, correl2)...
            - stk_distrib_normal_cdf(diff_to_thres_u, 0, 1).^2;
        % Remark: if correl2 = 0; futur_red = 0. However, there can be some
        % numerical instabilities which leads to a non-zero difference.
        
        if ~isempty(weights)
            red_val = WEIGHTS'.*red_val;
        end
    end
end
end
